import React, {Component} from 'react';
import R from 'ramda';
import {validateComponent} from '../../services/component-validate-service';
import {getValidators} from '../../services/component-service';
import {transformMessage} from '../../utils/transform-utils';
import TestComponent from './test-component';
import ElementNavTitle from './element-nav-title';
import kitchenTestComponentProperties from './kitchen-test-component-props';

export default class DipKitchenTestComponent extends Component {
    static propTypes = kitchenTestComponentProperties;

    static defaultProps = {
        options: {copyModel: true},
        showBindings: true,
        validators: []
    };

    constructor(props) {
        super(props);
        if (!R.isEmpty(props.validators)) {
            this.onModelChange(props.model, false);
        }
    }

    onModelChange = async (val, updatePristinity = true, changedParams = {pristine: false}) => {
        if (this.props.isForm) {
            return;
        }
        const {
            currentState,
            modelName,
            updateState,
            validators
        } = this.props;
        const {componentProps} = currentState[modelName];
        const props = {
            ...componentProps,
            ...changedParams
        };
        const metadata = {
            ...componentProps.metadata,
            ...changedParams.metadata
        };
        const propsPristine = R.propOr(true, 'pristine', componentProps);
        const pristine = updatePristinity ? changedParams.pristine : propsPristine;

        const getNewContext = async () => {
            const context = componentProps.context;
            if (!context) {
                return [{getValidators: () => []}, val];
            }
            const transformedModel = context.transformModel(val);
            const updatePropState = (newState) => {
                setTimeout(() => updateState(modelName, newState));
            };
            const computedContext = await context.computeContext(updatePropState, context, transformedModel);
            return [{
                ...context,
                ...computedContext
            }, transformedModel];
        };
        const [newContext, transformedModel] = await getNewContext();
        const {message, valid} = validateComponent(
            this.props,
            [...getValidators({metadata, validators}), ...newContext.getValidators(newContext)],
            val,
            pristine
        );
        const newComponentProps = {
            message: transformMessage(message, props.showMessages),
            model: transformedModel,
            pristine,
            valid
        };
        setTimeout(() => updateState(modelName, {componentProps: newComponentProps}));
    };

    render() {
        const {
            className,
            currentState,
            children,
            isForm,
            label,
            modelName,
            showBindings,
            updateState
        } = this.props;

        const model = currentState[modelName];

        const {
            api,
            bindings,
            componentProps,
            navigation: {
                title: navigationTitle,
                description: navigationDescription,
                type: navigationType
            }
        } = model;

        const mergeContext = (context) => {
            updateState(modelName, {componentProps: {context}});
        };

        const onUpdateProps = (changedParams, updatedBinding) => {
            const updatedBindings = R.map(R.when(
                R.propEq('name', updatedBinding.name),
                R.assoc('value', updatedBinding.value)
            ))(bindings);

            updateState(modelName, {
                ...this.props.isForm ? {
                    ...changedParams,
                    bindings: updatedBindings
                } : {
                    bindings: updatedBindings,
                    componentProps: changedParams
                }
            });
            this.onModelChange(componentProps.model, false, changedParams);
        };

        const register = (val) => {
            setTimeout(() => updateState(modelName, {...currentState[modelName], api: val}));
        };

        const resetPristinity = this.props.isForm ? undefined : () => {
            this.onModelChange(componentProps.model, true, {pristine: true});
        };

        const childrenWithProps = React.Children.map(children,
            (child) => React.cloneElement(child, {
                ...{context: {}, ...componentProps},
                mergeContext,
                onModelChange: this.onModelChange,
                register
            }));

        const showElement = componentProps.context || isForm;

        return showElement ? <div>
            <ElementNavTitle
                description={navigationDescription}
                modelName={modelName}
                title={navigationTitle}
                type={navigationType}
            />
            <h3>{label}</h3>
            <TestComponent
                api={api}
                bindings={bindings}
                className={className}
                componentProps={componentProps}
                modelName={modelName}
                resetPristinity={resetPristinity}
                showBindings={showBindings}
                updateProps={onUpdateProps}>
                {childrenWithProps}
            </TestComponent>
        </div> : null;
    }
}
