import R from 'ramda';

export const generateTree = (arity, depth, parentIndex = '', labelPrefix = 'Node') => {
    const generateId = (index) =>
        `${R.toLower(labelPrefix)}-${parentIndex}-${index + 1}--${depth}`;
    const generateLabel = (index) =>
        `${labelPrefix} ${parentIndex}${parentIndex ? '.' : ''}${index + 1}...${depth}`;
    const generateChildren = (index) =>
        generateTree(arity, depth - 1, `${parentIndex}${parentIndex ? '.' : ''}${index + 1}`, labelPrefix);

    const createLevel = (index) => ({
        id: generateId(index),
        name: generateLabel(index)
    });

    const generateOneLevelTree = () => R.addIndex(R.map)((item, index) => createLevel(index), R.repeat(null, arity));

    const generateMultiLevelTree = () => R.addIndex(R.map)((item, index) => ({
        children: generateChildren(index),
        id: generateId(index),
        name: generateLabel(index)
    }))(R.repeat(null, arity));

    return depth === 1 ? generateOneLevelTree() : generateMultiLevelTree();
};

export const generateLazyChildren = (node) => [{
    id: `${node.id}.1`,
    loadOnDemand: true,
    name: `Lazy-Child${node.state.path}`
}];
