import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import './test-component.less';
import {bindingTypes, ComponentBindings} from './bindings/component-bindings';

export default class TestComponent extends PureComponent {
    static propTypes = {
        api: PropTypes.object,
        bindings: PropTypes.arrayOf(PropTypes.shape({
            name: PropTypes.string.isRequired,
            type: PropTypes.oneOf(R.values(bindingTypes)).isRequired,
            value: PropTypes.any
        })),
        children: PropTypes.oneOfType([
            PropTypes.element,
            PropTypes.array
        ]),
        className: PropTypes.string,
        componentProps: PropTypes.object,
        modelName: PropTypes.string,
        resetPristinity: PropTypes.func,
        showBindings: PropTypes.bool,
        updateProps: PropTypes.func
    };

    render() {
        const {
            api,
            bindings,
            children,
            className,
            componentProps,
            modelName,
            resetPristinity,
            showBindings,
            updateProps
        } = this.props;

        return <div className={className}>
            <div className="testComponent">
                {children}
            </div>
            {
                showBindings && <ComponentBindings
                    api={api}
                    bindingItems={bindings}
                    componentProps={componentProps}
                    modelName={modelName}
                    resetPristinity={resetPristinity}
                    updateProps={updateProps}
                />
            }
        </div>;
    }
}
