import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import {XlReactWidgetTreeSelect} from '../../xl-react-widget/xl-react-widget-tree-select/xl-react-widget-tree-select';
import DisplayComponent from '../sink/display-component';
import ElementNavTitle from './element-nav-title';

export default class TreeSelectTestComponent extends PureComponent {
    static propTypes = {
        currentState: PropTypes.object.isRequired,
        modelName: PropTypes.string.isRequired,
        showBindings: PropTypes.bool
    };

    static defaultProps = {showBindings: false};

    constructor(props) {
        super(props);

        const {currentState, modelName} = props;
        const {
            data,
            model: {
                id: selectedId,
                title
            }
        } = currentState[modelName];

        this.state = {
            data,
            selectedId,
            title
        };
    }

    handleModelChange = ({selectedId, title}) => {
        this.setState({selectedId, title});
    };

    render() {
        const {currentState, modelName, showBindings} = this.props;
        const {data, selectedId, title} = this.state;
        const {description, title: navigationTitle, type} = currentState[modelName].navigation;

        const displayBindings = () => showBindings ? <div className="testBindings">
            <DisplayComponent
                className="displayTextBlock"
                label="Model"
                value={{selectedId, title}}
            />
            <DisplayComponent
                className="displayTextBlock"
                label="Folder Tree"
                value={data}
            />
        </div> : null;

        return <div>
            <ElementNavTitle
                description={description}
                modelName={modelName}
                title={navigationTitle}
                type={type}
            />
            <div className="testComponent">
                <div className="xl-react-widget-tree-select">
                    <XlReactWidgetTreeSelect
                        data={data}
                        onModelChange={this.handleModelChange}
                        selectedFolderId={selectedId}
                    />
                </div>
                {displayBindings()}
            </div>
        </div>;
    }
}
