import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import {XlReactWidgetCategory} from '../../xl-react-widget/xl-react-widget-category/xl-react-widget-category';
import ElementNavTitle from './element-nav-title';

export default class WidgetCategoryTestComponent extends PureComponent {
    static propTypes = {
        categoryName: PropTypes.string.isRequired,
        children: PropTypes.oneOfType([PropTypes.object, PropTypes.array]),
        className: PropTypes.string.isRequired,
        currentState: PropTypes.object.isRequired,
        modelName: PropTypes.string.isRequired,
        updateState: PropTypes.func.isRequired
    };

    toggleOpen = () => {
        const {currentState, modelName, updateState} = this.props;
        currentState[modelName] = updateState(modelName, {
            ...currentState[modelName],
            open: !currentState[modelName].open
        });
    };

    render() {
        const {categoryName, children, className, currentState, modelName} = this.props;
        const model = currentState[modelName];
        const {description, title, type} = model.navigation;

        return <div>
            <ElementNavTitle
                description={description}
                modelName={modelName}
                title={title}
                type={type}
            />
            <div className={className}>
                <div className="testComponent">
                    <XlReactWidgetCategory
                        categoryName={categoryName}
                        open={model.open}
                        toggleOpen={this.toggleOpen}>
                        {children}
                    </XlReactWidgetCategory>
                </div>
            </div>
        </div>;
    }
}
