import R from 'ramda';
import {widgetInitialState, widgetReduxTableData} from '../widget-page/widget-initial-state';

const timeout = 500;

export const fetchDataApi = (filter, paging, sorting) => {
    const mappedFilter = {...R.omit(['firstname-filter-key'], filter), firstname: filter['firstname-filter-key']};
    const mappedSorting = {...sorting, name: sorting.name === 'address-sort-key' ? 'address' : sorting.name};

    const {page, resultsPerPage} = paging;
    const {order, name} = mappedSorting;

    const sortFunc = R.sortWith([order === 'asc' ? R.ascend(R.prop(name)) : R.descend(R.prop(name))]);
    const sliceFunc = R.slice((page - 1) * resultsPerPage, page * resultsPerPage);

    const containsElementInList = (filterVal) => (value) => R.pipe(
        R.pluck('value'),
        R.cond([
            [R.complement(R.isEmpty), R.includes(value)],
            [R.T, R.T]
        ])
    )(filterVal);

    const containsString = (filterVal) => (value) => R.includes(R.toLower(filterVal))(R.toLower(value));

    const containsValue = (filterVal) => (value) => R.cond([
        [R.is(String), () => containsString(filterVal)(value)],
        [R.is(Array), () => containsElementInList(filterVal)(value)]
    ])(filterVal);

    const filterFunc = () => R.reduce(
        (acc, filterKey) =>
            mappedFilter[filterKey] ?
                R.filter(R.where({[filterKey]: containsValue(mappedFilter[filterKey])}))(acc) : acc,
        widgetReduxTableData,
        R.keys(mappedFilter)
    );

    const data = R.pipe(
        filterFunc,
        sortFunc
    )(widgetReduxTableData);

    return new Promise((resolve) => setTimeout(() => resolve({
        data: sliceFunc(data),
        headers: {'x-total-count': R.length(data)}
    }), timeout));
};

export const tableWithDatePickerResource = (filter) => new Promise((resolve) => setTimeout(() => {
    const data = widgetInitialState
        .widgetReduxTable3
        .data
        .filter((row) => row.dateOfBirth.isBetween(filter.afterDateOfBirth, filter.beforeDateOfBirth, null, '[]'));
    return resolve({
        data,
        headers: {'x-total-count': data.length}
    });
}, timeout));

export const tableWithToggleFilterResource = (filter, paging) => new Promise((resolve) => setTimeout(() => {
    const {page, resultsPerPage} = paging;

    const filterFunc = R.cond([
        [R.equals('lt50'), () => (obj) => obj.value < 50],
        [R.equals('gt50'), () => (obj) => obj.value > 50],
        [R.T, () => R.T]
    ])(filter.value);

    const sliceFunc = R.slice((page - 1) * resultsPerPage, page * resultsPerPage);

    const data = widgetInitialState
        .widgetReduxTable4
        .data
        .filter(filterFunc);

    return resolve({
        data: sliceFunc(data),
        headers: {'x-total-count': data.length}
    });
}, timeout));
