import React, {PureComponent} from 'react';
import {XlReactTabSystemContainer} from '../../xl-react-tab-system/xl-react-tab-system-container';

import TabsPlayground from '../playground/tabs-playground';
import {TabsInitialState} from './TabsInitialState';

export class Tabs extends PureComponent {
    constructor(props) {
        super(props);
        this.state = TabsInitialState;
    }

    handleAddTab = (section) => () => this.state[section].tabsApi.addTab({
        title: 'xl-react-widget-string',
        viewType: 'xl-react-widget-string'
    });

    handleOnNewTab = (section) => () => this.state[section].tabsApi.addTab({
        title: 'New Tab',
        viewType: 'xl-react-widget-string'
    });

    handleChangeActiveTabId = (section) => (newTabId) =>
        this.setState((current) => ({[section]: {...current[section], activeTabId: newTabId}}));

    handleChangeData = (section) => (newData) =>
        this.setState((current) => ({[section]: {...current[section], data: newData}}));

    handleSkinChange = (section) => (event) => {
        const newSkin = event.target.value;
        this.setState((current) => ({[section]: {...current[section], skin: newSkin}}));
    };

    handleToggleLoading = (section) => () => {
        const tabsApi = this.state[section].tabsApi;
        const {tabId, loading} = tabsApi.getActiveTab();
        tabsApi.changeLoading(tabId, !loading);
    };

    render() {
        return <div className="tabs-page">
            <div className="xl-react-tab-system-default">
                <h2 className="navigation">Tabs <span className="desc">xl-react-tab-system</span></h2>
                <XlReactTabSystemContainer
                    actions={this.state.XlReactTabSystem1.actions}
                    activeTabId={this.state.XlReactTabSystem1.activeTabId}
                    config={this.state.XlReactTabSystem1.config}
                    data={this.state.XlReactTabSystem1.data}
                    onChangeActiveTabId={this.handleChangeActiveTabId('XlReactTabSystem1')}
                    onChangeData={this.handleChangeData('XlReactTabSystem1')}
                    onNewTab={this.handleOnNewTab('XlReactTabSystem1')}
                    register={this.state.XlReactTabSystem1.registerApi}
                    skin={this.state.XlReactTabSystem1.skin}
                    tabDefinitions={this.state.XlReactTabSystem1.tabDefinitions}
                />

                <TabsPlayground
                    handlers={{
                        handleAddTab: this.handleAddTab('XlReactTabSystem1'),
                        handleSkinChange: this.handleSkinChange('XlReactTabSystem1'),
                        handleToggleLoading: this.handleToggleLoading('XlReactTabSystem1')
                    }}
                    skin={this.state.XlReactTabSystem1.skin}
                />
            </div>
        </div>;
    }
}
