// eslint-disable-line max-lines
import classNames from 'classnames';
import {Promise} from 'es6-promise';
import PropTypes from 'prop-types';
import R from 'ramda';
import RS from 'ramdasauce';
import React, {PureComponent} from 'react';
import {connect} from 'react-redux';
import {getBaseName} from '../../utils/ci-utils';
import {transformItemOr} from '../../utils/transform-utils';
import {XlReactWidgetAutocomplete}
    from '../../xl-react-widget/xl-react-widget-autocomplete/xl-react-widget-autocomplete';
import {XlReactWidgetConfirmationDialog}
    from '../../xl-react-widget/xl-react-widget-confirmation-dialog/xl-react-widget-confirmation-dialog';
import {XlReactWidgetModalDialogContainer}
    from '../../xl-react-widget/xl-react-widget-confirmation-dialog-container/confirmation-dialog-container';
import {XlReactWidgetDate} from '../../xl-react-widget/xl-react-widget-date/xl-react-widget-date';
import {XlReactWidgetFileUpload} from '../../xl-react-widget/xl-react-widget-file-upload/xl-react-widget-file-upload';
import {XlReactWidgetFilter} from '../../xl-react-widget/xl-react-widget-filter/xl-react-widget-filter';
import {XlReactWidgetInputAddon} from '../../xl-react-widget/xl-react-widget-input-addon/xl-react-widget-input-addon';
import {XlReactWidgetInputAddonGroup}
    from '../../xl-react-widget/xl-react-widget-input-addon/xl-react-widget-input-addon-group';
import {XlReactWidgetModalDialog}
    from '../../xl-react-widget/xl-react-widget-modal-dialog/xl-react-widget-modal-dialog';
import {XlReactWidgetProgressbar} from '../../xl-react-widget/xl-react-widget-progressbar/xl-react-widget-progressbar';
import {XlReactWidgetRadioGroup} from '../../xl-react-widget/xl-react-widget-radio-group/xl-react-widget-radio-group';
import {XlReactWidgetString} from '../../xl-react-widget/xl-react-widget-string/xl-react-widget-string';
import {XlReactWidgetSwitch} from '../../xl-react-widget/xl-react-widget-switch/xl-react-widget-switch';
import {XlReactWidgetSwitchButton}
    from '../../xl-react-widget/xl-react-widget-switch/xl-react-widget-switch-button/xl-react-widget-switch-button';
import {XlReactWidgetTags} from '../../xl-react-widget/xl-react-widget-tags/xl-react-widget-tags';
import {XlReactWidgetTime} from '../../xl-react-widget/xl-react-widget-time/xl-react-widget-time';
import {XlReactWidgetWithPopup} from '../../xl-react-widget/xl-react-widget-with-popup/xl-react-widget-with-popup';
import {xlReactTreeRowRenderer} from '../../xl-react-widget/xl-react-widget-tree/row/xl-react-tree-row-renderer';
import {XlReactWidgetTree} from '../../xl-react-widget/xl-react-widget-tree/xl-react-widget-tree';
import {XlReactWidgetLazyLoadedTree}
    from '../../xl-react-widget/xl-react-widget-lazy-loaded-tree/xl-react-widget-lazy-loaded-tree';
import {XlReactWidgetMonacoDiffViewer}
    from '../../xl-react-widget/xl-react-widget-monaco/xl-react-widget-monaco-diff-viewer/xl-react-widget-monaco-diff-viewer'; // eslint-disable-line max-len
import kitchenSinkActions from '../actions/kitchen-sink-actions';
import widgetActions from '../actions/widget-actions';
import {getHandlers} from '../handlers/predefined-tags-autocomplete-handlers';
import AlertMessageTestComponent from '../playground/alert-message-test-component';
import {ComponentNavigator} from '../playground/component-navigator/component-navigator';
import WidgetKitchenTestComponent from '../playground/widget-kitchen-test-component';
import {ReduxTableTestComponent} from '../playground/redux-table-test-component';
import {SpinnerTestComponent} from '../playground/spinner-test-component';
import {ORIENTATION, SplitPaneTestComponent} from '../playground/split-pane-test-component';
import {TableTestComponent} from '../playground/table-test-component';
import TreeSelectTestComponent from '../playground/tree-select-test-component';
import TreeTestComponent from '../playground/tree-test-component';
import WidgetCategoryTestComponent from '../playground/widget-category-test-component';
import {WidgetStringWithButtonAddonTestComponent} from '../playground/widget-string-with-button-addon-test-component';
import {XlReactWidgetMonacoEditor}
    from '../../xl-react-widget/xl-react-widget-monaco/xl-react-widget-monaco-editor/xl-react-widget-monaco-editor';
import {XlLastTimeUpdatedComponent}
    from '../../xl-react-widget/xl-react-widget-last-time-updated/last-updated-component';
import {
    fetchDataApi,
    tableWithDatePickerResource,
    tableWithToggleFilterResource
} from '../resource/redux-table-resource';
import DisplayTestComponent from '../playground/display-test-component';
import WidgetPatchTestEditor from './widget-patch-test-editor/wiget-patch-test-editor';
import WidgetPatchTestSample from './widget-patch-test-sample/widget-patch-test-sample';
import {autocompleteElements} from './widget-initial-state';

import './widget-page.less';

const tableId = 'redux-table-1';
const tableId2 = 'redux-table-2';
const tableId3 = 'redux-table-3';
const tableId4 = 'redux-table-4';
const tableId5 = 'redux-table-5';

const mapStateToProps = (state) => ({
    reduxTableInitialized: !RS.isNilOrEmpty(state.tables[tableId]),
    widgetContext: state.widget
});

const mapDispatchToProps = {
    initializeReduxTable: kitchenSinkActions.initializeReduxTable,
    showConfirmationDialog: kitchenSinkActions.openConfirmationDialog,
    updateModelState: widgetActions.updateModelState
};

@connect(mapStateToProps, mapDispatchToProps)
export class Widget extends PureComponent {
    static propTypes = {
        actions: PropTypes.object,
        initializeReduxTable: PropTypes.func,
        match: PropTypes.object,
        reduxTableInitialized: PropTypes.bool,
        showConfirmationDialog: PropTypes.func,
        updateModelState: PropTypes.func,
        widgetAlert1Type: PropTypes.string,
        widgetContext: PropTypes.object
    };

    updateState = (modelName, newState) => {
        this.props.updateModelState(modelName, newState);
    };

    connectComponent = ({modelName, modelChangeKey = 'onModelChange', modelKey = 'model'}) => ({
        [modelChangeKey]: (value) => this.updateState(modelName, {componentProps: {[modelKey]: value}}),
        [modelKey]: R.path(['props', 'widgetContext', modelName, 'componentProps', modelKey], this)
    });

    render() {
        const {match} = this.props;

        const autocompleteHandler = ({value}) => {
            const term = transformItemOr('', value, 'id');
            const filterCond = (item) => R.toLower(item.id)
                .indexOf(R.toLower(term)) !== -1;
            const res = R.pipe(
                R.filter(filterCond),
                R.sortBy(
                    R.compose(
                        R.toLower,
                        R.prop('id')
                    )
                )
            )(autocompleteElements);
            return Promise.resolve(res);
        };

        const delayedAutocompleteHandler = (valueModel) =>
            autocompleteHandler(valueModel)
                .then((suggestions) => new Promise(
                    (resolve) => setTimeout(() => resolve(suggestions), 10000)
                ));

        const autocompleteHandlers = {addCandidates: autocompleteHandler};
        const delayedAutocompleteHandlers = {addCandidates: delayedAutocompleteHandler};

        const widgets = {
            widgetAlert1: <AlertMessageTestComponent
                className={'widgetAlert1'}
                currentState={this.props.widgetContext}
                key={'widgetAlert1'}
                modelName="widgetAlert1"
                updateState={this.updateState}
            />,
            widgetAutocomplete1: <WidgetKitchenTestComponent
                className={'xl-react-widget-autocomplete-no-default-value'}
                currentState={this.props.widgetContext}
                key={'widgetAutocomplete1'}
                label={'Autocomplete (string)'}
                modelName={'widgetAutocomplete1'}
                updateState={this.updateState}>
                <XlReactWidgetAutocomplete
                    handlers={autocompleteHandlers}
                />
            </WidgetKitchenTestComponent>,
            widgetAutocomplete2: <WidgetKitchenTestComponent
                className={'xl-react-widget-autocomplete-width-default-value'}
                currentState={this.props.widgetContext}
                key={'widgetAutocomplete2'}
                label={'Autocomplete (string) - with default value'}
                modelName={'widgetAutocomplete2'}
                updateState={this.updateState}>
                <XlReactWidgetAutocomplete
                    handlers={autocompleteHandlers}
                />
            </WidgetKitchenTestComponent>,
            widgetAutocomplete3: <WidgetKitchenTestComponent
                className={'xl-react-widget-autocomplete-width-default-object-value'}
                currentState={this.props.widgetContext}
                key={'widgetAutocomplete3'}
                label={'Autocomplete (string) - with default object value'}
                modelName={'widgetAutocomplete3'}
                updateState={this.updateState}>
                <XlReactWidgetAutocomplete
                    handlers={autocompleteHandlers}
                    objLabel={'id'}
                />
            </WidgetKitchenTestComponent>,
            widgetAutocomplete4: <WidgetKitchenTestComponent
                className={'xl-react-widget-autocomplete-with-addons'}
                currentState={this.props.widgetContext}
                key={'widgetAutocomplete4'}
                label={'Autocomplete - with input addons'}
                modelName={'widgetAutocomplete4'}
                updateState={this.updateState}>
                <XlReactWidgetAutocomplete
                    addons={
                        <XlReactWidgetInputAddonGroup>
                            <XlReactWidgetInputAddon>
                                <i className="xl-icon keyboard-icon"/>
                            </XlReactWidgetInputAddon>
                            <XlReactWidgetInputAddon>
                                2nd addon
                            </XlReactWidgetInputAddon>
                        </XlReactWidgetInputAddonGroup>
                    }
                    handlers={autocompleteHandlers}
                    objLabel={'id'}
                />
            </WidgetKitchenTestComponent>,
            widgetAutocomplete5: <WidgetKitchenTestComponent
                className={'xl-react-widget-autocomplete-delayed'}
                currentState={this.props.widgetContext}
                key={'widgetAutocomplete5'}
                label={'Autocomplete (string)'}
                modelName={'widgetAutocomplete5'}
                updateState={this.updateState}>
                <XlReactWidgetAutocomplete
                    handlers={delayedAutocompleteHandlers}
                />
            </WidgetKitchenTestComponent>,
            widgetBottomSplitPane: <SplitPaneTestComponent
                className={'xl-react-widget-bottom-split-pane'}
                currentState={this.props.widgetContext}
                key={'widgetBottomSplitPane'}
                modelName={'widgetBottomSplitPane'}
                orientation={ORIENTATION.HORIZONTAL}
                updateState={this.updateState}>
                <section>
                    Main pane part
                </section>
                <section>
                    Bottom pane part
                </section>
            </SplitPaneTestComponent>,
            widgetCategory: <WidgetCategoryTestComponent
                categoryName={'Panel'}
                className="xl-react-widget-category-default"
                currentState={this.props.widgetContext}
                key={'widgetCategory'}
                modelName="widgetCategory"
                updateState={this.updateState}>
                <span>Panel context</span>
            </WidgetCategoryTestComponent>,
            widgetConfirmationDialog1: <WidgetKitchenTestComponent
                className={'xl-react-widget-confirmation-dialog-default'}
                currentState={this.props.widgetContext}
                key={'widgetConfirmationDialog1'}
                modelName={'widgetConfirmationDialog1'}
                updateState={this.updateState}>
                <XlReactWidgetConfirmationDialog>
                    <p className="confirmation-dialog-text">
                        {this.props.widgetContext.widgetConfirmationDialog1.text}
                    </p>
                </XlReactWidgetConfirmationDialog>
            </WidgetKitchenTestComponent>,
            widgetConfirmationDialogContainer: <WidgetKitchenTestComponent
                className={'xl-react-widget-confirmation-dialog-container-default'}
                currentState={this.props.widgetContext}
                key={'widgetConfirmationDialogContainer'}
                modelName={'widgetConfirmationDialogContainer'}
                showBindings={true}
                updateState={this.updateState}>
                <XlReactWidgetModalDialogContainer/>
            </WidgetKitchenTestComponent>,
            widgetDate1: <WidgetKitchenTestComponent
                className={'xl-react-widget-date-default'}
                currentState={this.props.widgetContext}
                key={'widgetDate1'}
                label={'Datepicker'}
                modelName={'widgetDate1'}
                updateState={this.updateState}>
                <XlReactWidgetDate/>
            </WidgetKitchenTestComponent>,
            widgetFileUpload1: <WidgetKitchenTestComponent
                className={'xl-react-widget-file-upload'}
                currentState={this.props.widgetContext}
                disabledToggles={['auto-focus']}
                key={'widgetFileUpload1'}
                label={'Upload file widget'}
                modelName={'widgetFileUpload1'}
                updateState={this.updateState}>
                <XlReactWidgetFileUpload/>
            </WidgetKitchenTestComponent>,
            widgetFilter1: <WidgetKitchenTestComponent
                className={'xl-react-widget-filter'}
                currentState={this.props.widgetContext}
                key={'widgetFilter1'}
                label={'My filter'}
                modelName={'widgetFilter1'}
                showBindings={false}
                updateState={this.updateState}>
                <XlReactWidgetFilter/>
            </WidgetKitchenTestComponent>,
            widgetLastTimeUpdated: <DisplayTestComponent
                className={'xl-react-widget-last=time-update'}
                currentState={this.props.widgetContext}
                key={'widgetLastTimeUpdated'}
                modelName={'widgetLastTimeUpdated'}>
                <XlLastTimeUpdatedComponent value={this.props.widgetContext.widgetLastTimeUpdated.value}/>
            </DisplayTestComponent>,
            widgetModalDialog1: <WidgetKitchenTestComponent
                className={'xl-react-widget-modal-dialog-default'}
                currentState={this.props.widgetContext}
                key={'widgetModalDialog1'}
                modelName={'widgetModalDialog1'}
                updateState={this.updateState}>
                <XlReactWidgetModalDialog>
                    <p className="widget-dialog-text">{this.props.widgetContext.widgetModalDialog1.text}</p>
                </XlReactWidgetModalDialog>
            </WidgetKitchenTestComponent>,
            widgetModalDialog2: <WidgetKitchenTestComponent
                className={'xl-react-widget-modal-dialog-default'}
                currentState={this.props.widgetContext}
                key={'widgetModalDialog2'}
                modelName={'widgetModalDialog2'}
                updateState={this.updateState}>
                <XlReactWidgetModalDialog>
                    <p className="widget-dialog-text">{this.props.widgetContext.widgetModalDialog2.text}</p>
                </XlReactWidgetModalDialog>
            </WidgetKitchenTestComponent>,
            widgetMonacoDiffViewer: <DisplayTestComponent
                className={'xl-react-widget-monaco-diff-viewer'}
                currentState={this.props.widgetContext}
                key={'widgetMonacoDiffViewer'}
                modelName={'widgetMonacoDiffViewer'}>
                <XlReactWidgetMonacoDiffViewer
                    options={{readOnly: true}}
                    original={this.props.widgetContext.widgetMonacoDiffViewer.original}
                    value={this.props.widgetContext.widgetMonacoDiffViewer.value}
                />
            </DisplayTestComponent>,
            widgetMonacoEditor: <WidgetKitchenTestComponent
                className={'xl-react-widget-monaco-editor'}
                currentState={this.props.widgetContext}
                key={'widgetMonacoEditor'}
                modelName={'widgetMonacoEditor'}
                updateState={this.updateState}>
                <XlReactWidgetMonacoEditor
                    {...this.connectComponent({
                        modelChangeKey: 'onChange',
                        modelKey: 'value',
                        modelName: 'widgetMonacoEditor'
                    })}

                />
            </WidgetKitchenTestComponent>,
            widgetPatchEditor: <WidgetKitchenTestComponent
                className={'xl-react-widget-patch-editor'}
                currentState={this.props.widgetContext}
                key={'widgetPatchEditor'}
                modelName={'widgetPatchEditor'}
                updateState={this.updateState}>
                <WidgetPatchTestEditor
                    modelName={'widgetPatchEditor'}
                    updateState={this.updateState}
                />
            </WidgetKitchenTestComponent>,
            widgetPatchSample: <WidgetKitchenTestComponent
                className={'widget-patch-test-sample-default'}
                currentState={this.props.widgetContext}
                key={'widgetPatchSample'}
                modelName={'widgetPatchSample'}
                updateState={this.updateState}>
                <WidgetPatchTestSample
                    modelName={'widgetPatchSample'}
                    updateState={this.updateState}
                />
            </WidgetKitchenTestComponent>,
            widgetProgress1: <WidgetKitchenTestComponent
                className={'xl-react-widget-progressbar-default'}
                currentState={this.props.widgetContext}
                disabledToggles={['auto-focus', 'disabled', 'required', 'show-errors']}
                key={'widgetProgress1'}
                label={'With default values'}
                modelName={'widgetProgress1'}
                updateState={this.updateState}>
                <XlReactWidgetProgressbar/>
            </WidgetKitchenTestComponent>,
            widgetRadioGroup1: <WidgetKitchenTestComponent
                className="xl-react-widget-radio-group-default"
                currentState={this.props.widgetContext}
                key={'widgetRadioGroup1'}
                modelName="widgetRadioGroup1"
                showBindings={true}
                updateState={this.updateState}>
                <XlReactWidgetRadioGroup name="Group" separatorLabel="or"/>
            </WidgetKitchenTestComponent>,
            widgetReduxTable: <ReduxTableTestComponent
                currentState={this.props.widgetContext}
                fetchDataApi={fetchDataApi}
                initializeReduxTable={this.props.initializeReduxTable}
                key={'widgetReduxTable'}
                modelName={'widgetReduxTable'}
                showBindings={true}
                tableId={tableId}
                tableInitialized={this.props.reduxTableInitialized}
                updateState={this.updateState}
            />,
            widgetReduxTable2: <ReduxTableTestComponent
                currentState={this.props.widgetContext}
                fetchDataApi={fetchDataApi}
                initializeReduxTable={this.props.initializeReduxTable}
                key={'widgetReduxTable2'}
                modelName={'widgetReduxTable2'}
                showBindings={false}
                tableId={tableId2}
                tableInitialized={this.props.reduxTableInitialized}
                updateState={this.updateState}
            />,
            widgetReduxTable3: <ReduxTableTestComponent
                currentState={this.props.widgetContext}
                fetchDataApi={tableWithDatePickerResource}
                initializeReduxTable={this.props.initializeReduxTable}
                key={'widgetReduxTable3'}
                modelName={'widgetReduxTable3'}
                showBindings={false}
                tableId={tableId3}
                tableInitialized={this.props.reduxTableInitialized}
                updateState={this.updateState}
            />,
            widgetReduxTable4: <ReduxTableTestComponent
                currentState={this.props.widgetContext}
                fetchDataApi={tableWithToggleFilterResource}
                initializeReduxTable={this.props.initializeReduxTable}
                key={'widgetReduxTable4'}
                modelName={'widgetReduxTable4'}
                showBindings={false}
                tableId={tableId4}
                tableInitialized={this.props.reduxTableInitialized}
                updateState={this.updateState}
            />,
            widgetReduxTable5: <ReduxTableTestComponent
                currentState={this.props.widgetContext}
                fetchDataApi={tableWithToggleFilterResource}
                initializeReduxTable={this.props.initializeReduxTable}
                key={'widgetReduxTable5'}
                modelName={'widgetReduxTable5'}
                showBindings={false}
                tableId={tableId5}
                tableInitialized={this.props.reduxTableInitialized}
                updateState={this.updateState}
            />,
            widgetSpinner: <SpinnerTestComponent
                className={'xl-react-widget-spinner-test'}
                currentState={this.props.widgetContext}
                key={'widgetSpinner'}
                modelName={'widgetSpinner'}
                showBindings={true}
                updateState={this.updateState}
            />,
            widgetString1: <WidgetKitchenTestComponent
                className={'xl-react-widget-string-default'}
                currentState={this.props.widgetContext}
                key={'widgetString1'}
                label={'Input String'}
                modelName={'widgetString1'}
                updateState={this.updateState}>
                <XlReactWidgetString/>
            </WidgetKitchenTestComponent>,
            widgetString2: <WidgetKitchenTestComponent
                className={'xl-react-widget-string-with-addons'}
                currentState={this.props.widgetContext}
                key={'widgetString2'}
                label={'Input String with addon'}
                modelName={'widgetString2'}
                updateState={this.updateState}>
                <XlReactWidgetString
                    addons={
                        <XlReactWidgetInputAddonGroup>
                            <XlReactWidgetInputAddon>
                                <i className="xl-icon keyboard-icon"/>
                            </XlReactWidgetInputAddon>
                            <XlReactWidgetInputAddon>
                                2nd addon
                            </XlReactWidgetInputAddon>
                        </XlReactWidgetInputAddonGroup>
                    }
                />
            </WidgetKitchenTestComponent>,
            widgetStringWithButtonAddon: <WidgetStringWithButtonAddonTestComponent
                className={'xl-react-widget-string-with-button-addon'}
                currentState={this.props.widgetContext}
                key={'widgetStringWithButtonAddon'}
                label={'Input String with addon'}
                modelName={'widgetStringWithButtonAddon'}
                updateState={this.updateState}
            />,
            widgetSwitch: <WidgetKitchenTestComponent
                currentState={this.props.widgetContext}
                key={'widgetSwitch'}
                label={'Switch button component'}
                modelName={'widgetSwitch'}
                updateState={this.updateState}>
                <XlReactWidgetSwitch
                    model={this.props.widgetContext.widgetSwitch.componentProps.model}
                    name="switch"
                    renderSwitch={
                        ({className}) =>
                            <XlReactWidgetSwitchButton className={className}>
                                Toggle switch
                            </XlReactWidgetSwitchButton>
                    }
                />
            </WidgetKitchenTestComponent>,
            widgetTable: <TableTestComponent
                currentState={this.props.widgetContext}
                key={'widgetTable'}
                modelName={'widgetTable'}
                showBindings={true}
                updateState={this.updateState}
            />,
            widgetTags1: <WidgetKitchenTestComponent
                className="xl-react-widget-tags"
                currentState={this.props.widgetContext}
                key={'widgetTags1'}
                label={'Tags'}
                modelName={'widgetTags1'}
                updateState={this.updateState}>
                <XlReactWidgetTags/>
            </WidgetKitchenTestComponent>,
            widgetTags2: <WidgetKitchenTestComponent
                className="xl-react-widget-tags"
                currentState={this.props.widgetContext}
                key={'widgetTags2'}
                label={'Tags with custom default error'}
                modelName={'widgetTags2'}
                updateState={this.updateState}>
                <XlReactWidgetTags/>
            </WidgetKitchenTestComponent>,
            widgetTags3: <WidgetKitchenTestComponent
                className="xl-react-widget-tags-with-autocomplete"
                currentState={this.props.widgetContext}
                key={'widgetTags3'}
                label={'Tags with Autocomplete input'}
                modelName={'widgetTags3'}
                updateState={this.updateState}>
                <XlReactWidgetTags
                    className="custom-tags-class"
                    handlers={{widgetTags3: getHandlers(this.props.widgetContext.widgetTags3.componentProps.model)}}
                    inputType="autocomplete"
                    objLabel="value"
                    tagTransform={getBaseName}
                />
            </WidgetKitchenTestComponent>,
            widgetTime1: <WidgetKitchenTestComponent
                className={'xl-react-widget-time'}
                currentState={this.props.widgetContext}
                key={'widgetTime1'}
                label={'Timepicker'}
                modelName={'widgetTime1'}
                updateState={this.updateState}>
                <XlReactWidgetTime/>
            </WidgetKitchenTestComponent>,
            widgetTree1: <TreeTestComponent
                className={'xl-react-widget-tree'}
                currentState={this.props.widgetContext}
                key={'widgetTree1'}
                modelName="widgetTree1"
                showBindings={true}
                updateState={this.updateState}>
                <XlReactWidgetTree rowRenderer={xlReactTreeRowRenderer}/>
            </TreeTestComponent>,
            widgetTree2: <TreeTestComponent
                className={'xl-react-widget-tree-lazy'}
                currentState={this.props.widgetContext}
                key={'widgetTree2'}
                modelName="widgetTree2"
                showBindings={true}
                updateState={this.updateState}>
                <XlReactWidgetTree rowRenderer={xlReactTreeRowRenderer}/>
            </TreeTestComponent>,
            widgetTreeLazyLoaded: <DisplayTestComponent
                className={'xl-react-widget-tree-lazy-loaded'}
                currentState={this.props.widgetContext}
                key={'widgetTreeLazyLoaded'}
                modelName="widgetTreeLazyLoaded">
                <XlReactWidgetLazyLoadedTree
                    hasMore={true}
                    isLoadingNewNodes={this.props.widgetContext.widgetTreeLazyLoaded.isLoadingNewNodes}
                    newNodes={this.props.widgetContext.widgetTreeLazyLoaded.model}
                    onScrollEnd={() => {
                        this.updateState('widgetTreeLazyLoaded', {isLoadingNewNodes: true});
                        setTimeout(() => {
                            this.updateState('widgetTreeLazyLoaded', {
                                isLoadingNewNodes: false,
                                model: R.clone(this.props.widgetContext.widgetTreeLazyLoaded.model)
                            });
                        }, 500);
                    }}
                    rowRenderer={xlReactTreeRowRenderer}
                />
            </DisplayTestComponent>,
            widgetTreeSelect: <TreeSelectTestComponent
                currentState={this.props.widgetContext}
                key={'widgetTreeSelect'}
                modelName="widgetTreeSelect"
                showBindings={true}
            />,
            widgetVerticalSplitPane: <SplitPaneTestComponent
                className={'xl-react-widget-vertical-split-pane'}
                currentState={this.props.widgetContext}
                key={'widgetVerticalSplitPane'}
                modelName={'widgetVerticalSplitPane'}
                orientation={ORIENTATION.VERTICAL}
                updateState={this.updateState}>
                <section>
                    Left pane part
                </section>
                <section>
                    Main pane part
                </section>
            </SplitPaneTestComponent>,
            widgetWithPopover: <DisplayTestComponent
                className={'xl-react-widget-vertical-split-pane'}
                currentState={this.props.widgetContext}
                key={'widgetWithPopover'}
                modelName={'widgetWithPopover'}>
                <XlReactWidgetWithPopup
                    delayHide={0}
                    delayShow={0}
                    popoverContent={this.props.widgetContext.widgetWithPopover.popoverContent}
                    popoverid={'widget-with-popover-1'}>
                    <div>Click on me</div>
                </XlReactWidgetWithPopup>
            </DisplayTestComponent>
        };

        const isSingleItem = match && widgets[match.params.name];

        return <div className={classNames('widget-page', {'single-item': isSingleItem})}>
            <ComponentNavigator componentType="widget" state={this.props.widgetContext}/>
            <div className="page-content">
                {isSingleItem ? widgets[match.params.name] : R.values(widgets)}
            </div>
        </div>;
    }
}

// eslint-enable-line max-lines
