import {useEffect, useRef} from 'react';

export const useDebouncedFunction = (func, wait) => {
    const argsRef = useRef();
    const timeout = useRef();

    function cleanup() {
        if (timeout.current) {
            clearTimeout(timeout.current);
        }
    }

    useEffect(() => cleanup, []);

    return (...args) => {
        argsRef.current = args;
        cleanup();

        timeout.current = setTimeout(() => {
            if (argsRef.current) {
                func(...argsRef.current);
            }
        }, wait);
    };
};
