import R from 'ramda';
import PropTypes from 'prop-types';
import React, {useRef} from 'react';
import {useBeforeFirstRender} from '../../utils/hook-utils';
import {isComponentDisabled} from '../../utils/components-utils';
import {invokeCallback} from '../../utils/function-utils';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {localeMessages} from '../../services/locale-service';
import {ERROR} from '../../utils/ci-utils';
import {XlReactDipIntegerComponent} from './xl-react-dip-integer-component';

export const INT_PATTERN_EXPRESSION = /^-?\d+$|^$/;

export const getInvalidIntegerValidators = (dateFormat, msg) => [(model) => {
    const message = {
        show: true,
        text: msg || localeMessages.invalidIntegerProvidedMessage,
        type: ERROR
    };

    const condition = model && !INT_PATTERN_EXPRESSION.test(model);
    return {condition, message};
}];

export const XlReactDipInteger = (props) => {
    const inputField = useRef();

    useBeforeFirstRender(() => {
        const api = {focus: () => inputField.current.focus()};
        invokeCallback(props, {
            functionName: 'register',
            values: [api]
        });
    });

    const processChange = (newModel) => {
        if (INT_PATTERN_EXPRESSION.test(newModel) || R.equals('-', newModel)) {
            props.onModelChange(newModel);
        }
    };

    const onChange = (event) => {
        event.preventDefault();
        processChange(event.target.value);
    };

    const onErase = () => {
        processChange('');
    };

    const {
        message,
        model,
        metadata: {description, label, placeholder, required},
        valid
    } = props;

    return (
        <XlReactDipIntegerComponent
            {...{
                assignRef: inputField,
                description,
                disabled: isComponentDisabled(props),
                handleChange: onChange,
                handleErase: onErase,
                label,
                message,
                model,
                placeholder,
                required,
                valid
            }}
        />
    );
};

XlReactDipInteger.propTypes = {
    model: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
    ...XlReactDipContainerProps
};

XlReactDipInteger.defaultProps = {
    displayMode: 'edit',
    metadata: {},
    model: '',
    validators: []
};
