import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import RU from '../../utils/ramda-utils';
import {isDisabled} from '../../utils/components-utils';
import {transformItem} from '../../utils/transform-utils';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {XlReactDipListComponent} from './xl-react-dip-list-component';

export class XlReactDipList extends Component {
    static propTypes = {
        handlers: PropTypes.object,
        model: PropTypes.array,
        ...XlReactDipContainerProps
    };

    static defaultProps = {
        context: {typed: ''},
        displayMode: 'edit',
        metadata: {},
        objLabel: 'id',
        validators: []
    };

    constructor(props) {
        super(props);
        RU.invoke('register', [this.api], props);
    }

    handleRemoveSortableListItem = (itemKey) => {
        this.notify(R.remove(itemKey, 1, this.props.model));
    };

    onAddItem = () => {
        const {typed} = this.props.context;
        this.notify([...this.props.model, typed]);
        this.props.mergeContext({typed: ''});
    };

    onSuggestionSelection = (item) => {
        this.notify([...this.props.model, transformItem(item, this.props.objLabel)], () => this.childApi.erase());
    };

    onUserInput = (inputText) => {
        this.props.mergeContext({typed: inputText});
    };

    api = {focus: () => this.childApi.focus()};

    notify(model, customCallback = R.F) {
        this.props.onModelChange(model);
        setTimeout(() => customCallback(), 20);
    }

    registerChildApi = (childApi) => {
        this.childApi = childApi;
    };

    render() {
        const {
            context: {typed},
            displayMode,
            handlers,
            metadata,
            message,
            model,
            objLabel,
            onMessageChange,
            onValidityChange,
            valid
        } = this.props;
        const {description, label, name, required, validators} = metadata;

        const autocompleteMetadata = R.omit(['default', 'description', 'label', 'required'], metadata);
        const disabled = isDisabled(this.props.disabled, this.props.displayMode);

        return <XlReactDipListComponent {...({
            autocompleteMetadata,
            description,
            disabled,
            displayMode,
            handleAddItem: this.onAddItem,
            handlers: handlers && handlers[name],
            handleSuggestionSelection: this.onSuggestionSelection,
            handleUserInput: this.onUserInput,
            label,
            message,
            model,
            name,
            objLabel,
            onMessageChange,
            onRemoveSortableListItem: this.handleRemoveSortableListItem,
            onValidityChange,
            registerChildApi: this.registerChildApi,
            required,
            typed,
            valid,
            validators
        })}
        />;
    }
}
