import R from 'ramda';
import React from 'react';
import {Column} from 'react-virtualized';
import {localeMessages} from '../../../services/locale-service';
import {InputValue} from '../input-value';
import {MapValueInput} from '../map-value-input';
import {COLUMNS, NEW_VALUE_CELL_KEY} from '../map-table-utils';
import {headerRenderer} from './header-renderer';

export const tableValueColumnRenderer = (options) => {
    const {
        columnWidth,
        disabled,
        focusedValue,
        handleCloseButton,
        handleNewValueBlur,
        handleFocus,
        handleNewValueKeyDown,
        handleRemoveKey,
        handleOnPasswordPaste,
        handleRevealPassword,
        handleValueChange,
        inputType,
        newValue,
        passwordsCanBeRevealed,
        passwordsRevealed,
        showNewKeyValueRow,
        showRemoveButton
    } = options;

    const cellRenderer = (cellProps) => {
        const {rowIndex, cellData, rowData} = cellProps;

        const shouldDisplayInputValue = () => R.and(R.equals(rowIndex, 0), showNewKeyValueRow);

        if (shouldDisplayInputValue()) {
            return <InputValue
                disabled={disabled}
                handleCloseButton={handleCloseButton}
                handleFocus={handleFocus}
                handleNewValueBlur={handleNewValueBlur}
                handleNewValueKeyDown={handleNewValueKeyDown}
                handleRevealPassword={handleRevealPassword}
                inputType={inputType}
                model={newValue}
                revealPassword={Boolean(passwordsRevealed[NEW_VALUE_CELL_KEY])}
                revealPasswordIconVisible={Boolean(passwordsCanBeRevealed[NEW_VALUE_CELL_KEY])}
                shouldFocus={focusedValue === NEW_VALUE_CELL_KEY}
                showRemoveButton={showRemoveButton}
            />;
        }

        return !R.isNil(cellData) && <MapValueInput
            changedValue={cellData}
            disabled={disabled}
            handleFocus={handleFocus}
            handleOnPasswordPaste={handleOnPasswordPaste}
            handleRemoveKey={handleRemoveKey}
            handleRevealPassword={handleRevealPassword}
            handleValueChange={handleValueChange}
            inputType={inputType}
            itemKey={rowData.key}
            revealPassword={Boolean(passwordsRevealed[rowData.key])}
            revealPasswordIconVisible={Boolean(passwordsCanBeRevealed[rowData.key])}
            shouldFocus={focusedValue === rowData.key}
            value={cellData}
        />;
    };

    return <Column
        cellRenderer={cellRenderer}
        className="xl-virtualized-table-cell"
        dataKey="value"
        flexGrow={1}
        headerRenderer={headerRenderer(localeMessages.valueLabel, COLUMNS.value)}
        width={columnWidth}
    />;
};

