import R from 'ramda';
import React from 'react';
import PropTypes from 'prop-types';
import {Column} from 'react-virtualized';
import {localeMessages} from '../../../services/locale-service';
import {InputKey} from '../input-key';
import {InlineKeyInput} from '../../input/inline-key-input';
import {headerRenderer} from '../../xl-react-dip-map-string-string/columns/header-renderer';
import {shouldFocusInput} from '../../xl-react-dip-map-string-string/map-table-utils';

export const tableKeyColumn = (props) => {
    const {
        assignRef,
        cellState,
        columnWidth,
        disabled,
        handleCloseButton,
        handleKeyBlur,
        handleNewKeyBlur,
        handleNewKeyChange,
        handleNewKeyFocus,
        handleRemoveKey,
        invalid,
        newKey,
        showNewKeyValueRow,
        updateCellState
    } = props;

    /* eslint-disable complexity */
    const cellRenderer = (cellProps) => {
        const {rowIndex} = cellProps;

        const focusInput = shouldFocusInput(rowIndex, 'key', cellState);

        const handleUpdateCellState = (value) => {
            updateCellState({column: 'key', row: rowIndex, value});
        };

        const onFocusNewKey = (value, event) => {
            handleUpdateCellState(value);
            handleNewKeyFocus(event);
        };

        const shouldDisplayInputKey = () => R.and(R.equals(rowIndex, 0), showNewKeyValueRow);

        const getChangedValue = () => focusInput ? cellState.value : undefined;

        if (shouldDisplayInputKey()) {
            return <InputKey
                assignRef={assignRef}
                disabled={disabled}
                handleCloseButton={handleCloseButton}
                handleNewKeyBlur={handleNewKeyBlur}
                handleNewKeyChange={handleNewKeyChange}
                handleNewKeyFocus={onFocusNewKey}
                handleRemoveKey={handleRemoveKey}
                invalid={invalid}
                model={newKey}
                onBlur={handleKeyBlur}
                shouldFocus={focusInput}
                updateCellState={handleUpdateCellState}
                value={cellProps.rowData}
            />;
        }

        return !R.isNil(cellProps.rowData) && <InlineKeyInput
            changedValue={getChangedValue()}
            disabled={Boolean(disabled || !cellProps.rowData)}
            handleRemoveKey={handleRemoveKey}
            onBlur={handleKeyBlur}
            readOnly={disabled}
            shouldFocus={focusInput}
            showRemoveButton={true}
            updateCellState={handleUpdateCellState}
            value={cellProps.rowData}
        />;
    };
    /* eslint-enable complexity */

    return <Column
        cellRenderer={cellRenderer}
        className="xl-virtualized-table-cell"
        dataKey="key"
        flexGrow={1}
        headerRenderer={headerRenderer(localeMessages.nameLabel, 'key')}
        width={columnWidth}
    />;
};

tableKeyColumn.propTypes = {
    assignRef: PropTypes.func.isRequired,
    cellState: PropTypes.shape({
        column: PropTypes.oneOf(['key']).isRequired,
        row: PropTypes.number.isRequired,
        value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired
    }),
    columnWidth: PropTypes.number.isRequired,
    disabled: PropTypes.bool.isRequired,
    handleCloseButton: PropTypes.func.isRequired,
    handleKeyBlur: PropTypes.func.isRequired,
    handleNewKeyBlur: PropTypes.func.isRequired,
    handleNewKeyChange: PropTypes.func.isRequired,
    handleNewKeyFocus: PropTypes.func.isRequired,
    handleRemoveKey: PropTypes.func.isRequired,
    invalid: PropTypes.bool.isRequired,
    newKey: PropTypes.string,
    showNewKeyValueRow: PropTypes.bool.isRequired,
    updateCellState: PropTypes.func.isRequired,
    valid: PropTypes.bool.isRequired
};
