import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import R from 'ramda';
import {localeMessages} from '../../services/locale-service';
import RU from '../../utils/ramda-utils';
import {InlineKeyInput} from '../input/inline-key-input';

export class InputKey extends PureComponent {
    static propTypes = {
        assignRef: PropTypes.func.isRequired,
        changedValue: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
        disabled: PropTypes.bool.isRequired,
        handleCloseButton: PropTypes.func.isRequired,
        handleNewKeyBlur: PropTypes.func.isRequired,
        handleNewKeyChange: PropTypes.func.isRequired,
        handleNewKeyFocus: PropTypes.func.isRequired,
        handleRemoveKey: PropTypes.func.isRequired,
        invalid: PropTypes.bool.isRequired,
        model: PropTypes.string,
        onBlur: PropTypes.func,
        shouldFocus: PropTypes.bool,
        updateCellState: PropTypes.func,
        value: PropTypes.string.isRequired
    };

    constructor(props) {
        super(props);
        const {changedValue, value} = props;
        if (!R.isNil(changedValue) && value !== changedValue) {
            this.state = {internalValue: props.changedValue};
        } else {
            this.state = {internalValue: props.value};
        }
    }

    handleValueChange = (event) => {
        RU.invoke('onChange', [event], this.props);
        this.setState({internalValue: event.target.value}, () => {
            this.props.handleNewKeyFocus(this.state.internalValue);
        });
    };

    assignRef = (input) => {
        this.inputField = input;
        RU.invoke('assignRef', [input], this.props);
    };

    render() {
        const {
            assignRef,
            invalid,
            disabled,
            handleNewKeyFocus,
            handleNewKeyChange,
            handleNewKeyBlur,
            handleRemoveKey,
            model,
            shouldFocus
        } = this.props;

        const inputKeySpan = <span className="input-key">{localeMessages.keyPlaceholder}</span>;

        const customClasses = classNames('key-div', 'new-key-div', {invalid});
        const inputKeyInput = <div className={customClasses}>
            <InlineKeyInput
                assignRef={assignRef}
                disabled={disabled}
                handleRemoveKey={handleRemoveKey}
                name="add-key"
                onBlur={handleNewKeyBlur}
                onChange={handleNewKeyChange}
                placeholder={localeMessages.keyPlaceholder}
                readOnly={disabled}
                shouldFocus={shouldFocus}
                showRemoveButton={true}
                updateCellState={handleNewKeyFocus}
                value={model}
            />
        </div>;

        return disabled ? inputKeySpan : inputKeyInput;
    }
}
