import R from 'ramda';
import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import RU from '../utils/ramda-utils';
import {XlReactDipLookup} from '../xl-react-dip/xl-react-dip-lookup/xl-react-dip-lookup';

export class XlReactDipField extends PureComponent {
    static propTypes = {
        allowLookupValues: PropTypes.bool,
        fieldComponent: PropTypes.func.isRequired,
        model: PropTypes.any,
        onLookupToggle: PropTypes.func
    };

    handleLookupToggle = () => {
        const {model} = this.props;
        const name = R.path(['metadata', 'name'], this.props);
        RU.invoke('onLookupToggle', [name, !this.isLookUp(model), model], this.props);
    };

    isLookUp = (model) => model && R.propEq('kind', 'lookup', model);

    renderLookupToggle = (props) => <XlReactDipLookup
        {...props}
        onLookupToggle={this.handleLookupToggle}
    />;

    renderComponent = R.cond([
        [R.allPass([
            R.propEq('allowLookupValues', true),
            R.pathEq(['metadata', 'password'], true),
            R.pathEq(['metadata', 'kind'], 'STRING')
        ]), this.renderLookupToggle],
        [R.T, ({fieldComponent: FieldComponent, ...rest}) => <FieldComponent {...rest}/>]
    ]);

    render() {
        return this.renderComponent(this.props);
    }
}
