// eslint-disable-line max-lines
import PropTypes from 'prop-types';
import R from 'ramda';
import RS from 'ramdasauce';
import debounce from 'debounce';
import React, {Component} from 'react';
import $ from 'jquery';
import classNames from 'classnames';
import {XlReactWidgetSpinner} from '../xl-react-widget-spinner/xl-react-widget-spinner';
import RU from '../../utils/ramda-utils';
import {transformItemOr} from '../../utils/transform-utils';
import {calculateMenuHeight, shouldDisplayDropdownUp} from '../../utils/components-utils';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {validationMessageClasses} from '../../utils/css-utils';
import {XlReactFormItemControl} from '../../xl-react-form/xl-react-form-item-control';
import {reasonTypeOfAddingCandidate} from './reason-type-of-adding-candidate';
import {XlReactWidgetAutocompleteInput} from './components/xl-react-widget-autocomplete-input-component';
import {XlReactWidgetAutocompleteMenu} from './components/xl-react-widget-autocomplete-menu';
import {autocompleteSuggestionBehaviorPropsType} from './components/autocomplete-suggestion-behavior-type';
import {computeNewState} from './xl-react-widget-autocomplete-service';

import './xl-react-widget-autocomplete.less';

export const INPUT_DEBOUNCE_TIME = 300;

export class XlReactWidgetAutocompleteComponent extends Component {
    static propTypes = {
        addons: PropTypes.oneOfType([
            PropTypes.node,
            PropTypes.arrayOf(PropTypes.node)
        ]),
        assignRef: PropTypes.func,
        className: PropTypes.string,
        clearInputOnBlur: PropTypes.bool,
        clearSuggestions: PropTypes.func.isRequired,
        componentId: PropTypes.string,
        description: PropTypes.string,
        disabled: PropTypes.bool,
        handleGetSuggestions: PropTypes.func,
        handleSuggestionSelected: PropTypes.func,
        inputProps: PropTypes.object,
        label: PropTypes.string,
        loadingSuggestions: PropTypes.bool,
        menuHeight: PropTypes.number,
        message: xlMessageTypeProp,
        messageHandler: PropTypes.func,
        model: PropTypes.oneOfType([
            PropTypes.string,
            PropTypes.object
        ]),
        objLabel: PropTypes.string,
        onBlur: PropTypes.func,
        onChange: PropTypes.func,
        onErase: PropTypes.func,
        onValidityChange: PropTypes.func,
        optionHeight: PropTypes.number,
        placeholder: PropTypes.string,
        renderInput: PropTypes.func,
        renderSuggestion: PropTypes.func,
        required: PropTypes.bool,
        requiredDirection: PropTypes.oneOf(['left', 'right']),
        showSearchIcon: PropTypes.bool.isRequired,
        subActions: PropTypes.arrayOf(PropTypes.shape({
            action: PropTypes.func.isRequired,
            isDisabled: PropTypes.func.isRequired,
            label: PropTypes.string.isRequired
        })),
        suggestionBehavior: autocompleteSuggestionBehaviorPropsType,
        suggestions: PropTypes.array,
        valid: PropTypes.bool
    };

    state = {isCollapsed: true, isFocused: false, typing: false};

    UNSAFE_componentWillMount() {
        const {model, objLabel} = this.props;
        this.setState({inputValue: transformItemOr('', model, objLabel)});
        RU.invoke('register', [this.api], this.props);
    }

    componentDidMount() {
        document.addEventListener('mousedown', this.onDocumentMouseDown);
        this.props.assignRef(this);
    }

    UNSAFE_componentWillReceiveProps(nextProps) {
        computeNewState(this.props, nextProps, this);
    }

    componentWillUnmount() {
        document.removeEventListener('mousedown', this.onDocumentMouseDown);
    }

    handleShowAllOptions = () => {
        $(`div[id="${this.props.componentId}"] input`).focus();
    };

    handleOnBlur = (event) => {
        if (this.justClickedOnSuggestionsContainer) {
            this.input.focus();
            return;
        }
        if (!this.justSelectedSuggestion) {
            this.onBlur(event);
            this.setState({isCollapsed: true, isFocused: false});
        }
        this.setState({typing: false}, () => this.props.onBlur(event));
    };

    handleOnChange = (event) => {
        const newValue = event.target.value;
        const {model} = this.props;
        if (!R.is(Object, model)) {
            this.notifyOnChange(event, newValue);
        }
        this.setState({inputValue: newValue});
        this.debounceGettingSuggestions(newValue, reasonTypeOfAddingCandidate.onChange);
        RU.invoke('onChange', [event], this.props.inputProps);
    };

    handleKeyDown = (event) => {
        this.setState({typing: true});
        RU.invoke('onKeyDown', [event], this.props.inputProps);
    };

    handleOnSuggestionMouseDown = () => {
        this.justSelectedSuggestion = true;
    };

    onDocumentMouseDown = (event) => { // eslint-disable-line complexity
        this.justClickedOnSuggestionsContainer = false;
        let node = event.detail && event.detail.target || event.target;
        while (node !== null && node !== document) {
            if (node.getAttribute('data-suggestion-index') !== null) {
                return;
            }
            if (node === this.suggestionsContainer) {
                this.justClickedOnSuggestionsContainer = true;
                return;
            }
            node = node.parentNode;
        }
    };

    onClickErase = () => {
        this.setState({inputValue: ''});
        this.props.onErase();
    };

    onBlur = (event) => {
        const {clearInputOnBlur, clearSuggestions, inputProps} = this.props;
        this.setState({
            inputValue: clearInputOnBlur ? '' : this.state.inputValue,
            isCollapsed: true,
            isFocused: false
        }, clearSuggestions);
        RU.invoke('onBlur', [event], inputProps);
    };

    api = {erase: () => this.setState({inputValue: ''})};

    getMenuSuggestions = () => {
        const {suggestions} = this.props;
        const {isCollapsed, isFocused} = this.state;
        if (R.and(!isCollapsed, isFocused, this.willRenderSuggestions(this.props))) {
            return suggestions;
        }
        return [];
    };

    notifyOnChange = (event, newValue) => {
        const {onChange, model} = this.props;
        if (model !== newValue) {
            onChange(event, {newValue});
        }
    };

    revealSuggestions = () => {
        if (this.state.isCollapsed) {
            this.setState({isCollapsed: false, isFocused: true});
        }
    };

    closeSuggestions = () => {
        if (!this.state.isCollapsed) {
            this.setState({isCollapsed: true, isFocused: false});
        }
    };

    storeInputRef = (el) => {
        const {inputProps} = this.props;
        this.input = el;
        RU.invoke('ref', [el], inputProps);
    };

    storeSuggestionsContainerRef = (el) => {
        this.suggestionsContainer = el;
    };

    debounceGettingSuggestions = debounce(this.props.handleGetSuggestions, INPUT_DEBOUNCE_TIME);

    willRenderSuggestions = (props) => {
        const {suggestions = []} = props;
        return suggestions.length > 0;
    };

    buildComponentInput = () => {
        const {inputValue} = this.state;
        const {
            addons, renderInput, disabled, inputProps, placeholder, showSearchIcon, suggestionBehavior
        } = this.props;
        return renderInput ? renderInput({
            ...this.props.inputProps,
            addons,
            onBlur: this.handleOnBlur,
            onChange: this.handleOnChange,
            onFocus: suggestionBehavior.handleOnFocus(this),
            showSearchIcon
        }) : <XlReactWidgetAutocompleteInput
            addons={addons}
            assignInputRef={this.storeInputRef}
            disabled={disabled}
            handleErase={this.onClickErase}
            inputProps={inputProps}
            onInputBlur={this.handleOnBlur}
            onInputChange={this.handleOnChange}
            onInputFocus={suggestionBehavior.handleOnFocus(this)}
            onKeyDown={this.handleKeyDown}
            onShowAllOptions={this.handleShowAllOptions}
            placeholder={placeholder}
            showDropDownTriangle={true}
            showSearchIcon={showSearchIcon}
            value={inputValue}
        />;
    };

    renderSubActions = (subActions) =>
        R.map(({action, isDisabled, label}) =>
            <a
                className="xl-react-link"
                disabled={isDisabled(this.props.suggestions)}
                key={label}
                onClick={R.partial(action, [this.props.suggestions])}>
                {label}
            </a>
        )(subActions);

    renderSuggestionsContainer = (menu) => {
        const {
            menuHeight,
            optionHeight,
            subActions,
            suggestions,
            loadingSuggestions
        } = this.props;

        const isUp = () => this.state.isFocused &&
            this.input &&
            shouldDisplayDropdownUp(this.input,
                calculateMenuHeight(menuHeight, suggestions.length, optionHeight, !RS.isNilOrEmpty(subActions))
            );

        const controlClass = classNames({
            'xl-react-autocomplete-suggestion-list-container': true,
            'xl-react-autocomplete-suggestion-list-container--up': isUp()
        });

        return <div className={controlClass} ref={this.storeSuggestionsContainerRef}>
            {!RS.isNilOrEmpty(subActions) &&
            <div className="actions-row">{this.renderSubActions(subActions)}</div>
            }{menu}
            <XlReactWidgetSpinner
                overlay={suggestions.length > 0}
                show={loadingSuggestions}
            />
        </div>;
    };

    render() {
        const {
            className, componentId, menuHeight,
            message, optionHeight, addons, renderSuggestion,
            suggestionBehavior, valid
        } = this.props;
        const {inputValue} = this.state;

        const suggestions = this.getMenuSuggestions();
        const controlClass = classNames(
            {
                'focused': this.state.isFocused,
                'xl-react-widget-autocomplete': true
            },
            className,
            validationMessageClasses(valid, message)
        );

        const containerClasses = classNames(
            'xl-react-autocomplete-container',
            {'xl-react-widget-autocomplete-input-group': !R.isNil(addons)}
        );

        return <XlReactFormItemControl {...({...this.props, controlClass})}>
            <div id={componentId}>
                <div className={containerClasses}>
                    {addons}
                    <div className="xl-react-autocomplete-wrapper">
                        {this.buildComponentInput()}
                        <XlReactWidgetAutocompleteMenu
                            menuHeight={calculateMenuHeight(menuHeight, suggestions.length, optionHeight, false)}
                            onSuggestionClick={suggestionBehavior.handleOnSuggestionClick(this)}
                            onSuggestionMouseDown={this.handleOnSuggestionMouseDown}
                            optionHeight={optionHeight}
                            suggestionRenderer={renderSuggestion ?
                                renderSuggestion : suggestionBehavior.renderSuggestion(this)}
                            suggestions={suggestions}
                            suggestionsContainerRenderer={this.renderSuggestionsContainer}
                            value={inputValue}
                        />
                    </div>
                </div>
            </div>
        </XlReactFormItemControl>;
    }
}
