import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import {Modal} from 'react-bootstrap';
import classNames from 'classnames';

import {setTitleOnEllipsis} from '../../utils/tooltip';
import './xl-react-widget-modal-dialog.less';

export class XlReactWidgetModalDialog extends PureComponent {
    static propTypes = {
        animation: PropTypes.bool,
        children: PropTypes.oneOfType([PropTypes.object, PropTypes.array]),
        dialogClassName: PropTypes.string,
        expandable: PropTypes.bool,
        handleClose: PropTypes.func,
        showModal: PropTypes.bool,
        title: PropTypes.string
    };

    static defaultProps = {animation: false, expandable: false};

    constructor(props) {
        super(props);
        this.state = {expanded: false};
    }

    handleExpandClick = () => {
        this.setState({expanded: !this.state.expanded});
    };

    handleMouseEnter = (event) => setTitleOnEllipsis($(event.currentTarget));

    handleMouseOut = (event) => {
        event.currentTarget.title = '';
    };

    modalTitle = () => this.props.title ? <Modal.Title>
        <span
            className="title"
            onMouseEnter={this.handleMouseEnter}
            onMouseOut={this.handleMouseOut}>
            {this.props.title}
        </span>
    </Modal.Title> : null;

    closeButton = () => {
        const {expandable, handleClose} = this.props;
        const {expanded} = this.state;
        const isCloseButtonAvailable = !R.isNil(handleClose);
        const expandClassName = expanded ? 'fullscreen-exit-icon' : 'fullscreen-icon';
        return <div className="xl-icon">
            {isCloseButtonAvailable && <i className=" close-icon" onClick={handleClose}/>}
            {expandable && <i className={expandClassName} onClick={this.handleExpandClick}/>}
        </div>;
    };

    render() {
        const {
            animation,
            children,
            dialogClassName,
            handleClose,
            showModal = false
        } = this.props;

        const {expanded} = this.state;

        const modalHeader = <Modal.Header>
            {this.closeButton()}
            {this.modalTitle()}
        </Modal.Header>;

        const modalClassNames = classNames('xl-react-widget-modal-dialog', {'modal-expanded': expanded});

        return <div className={modalClassNames}>
            <div className="static-modal">
                <Modal
                    animation={animation}
                    bsClass={'xl-react-modal'}
                    className={modalClassNames}
                    dialogClassName={dialogClassName}
                    onHide={handleClose}
                    show={showModal}>
                    {modalHeader}
                    {children}
                </Modal>
            </div>
        </div>;
    }
}
