/* eslint-disable */
import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import {Collapse} from 'react-bootstrap';
import R from 'ramda';
import RU from '../../utils/ramda-utils';
import {XlReactToggleButton} from '../../xl-react-buttons/xl-react-toggle-button/xl-react-toggle-button';
import {XlReactWidgetLabel} from '../xl-react-widget-label/xl-react-widget-label';
import {tagsAutocompleteSkins} from '../xl-react-widget-tags/xl-react-widget-tags-skins';
import {XlReactWidgetAutocomplete} from '../xl-react-widget-autocomplete/xl-react-widget-autocomplete';
import {XlReactWidgetString} from '../xl-react-widget-string/xl-react-widget-string';
import {XlReactWidgetTags} from '../xl-react-widget-tags/xl-react-widget-tags';
import {tableFiltersType} from '../xl-react-widget-table/xl-react-widget-table-types';
import {XlReactWidgetDate} from '../xl-react-widget-date/xl-react-widget-date';
import {buildColumns} from './filter-element-position-utils';
import {xlReactWidgetTableFilterTypes} from './xl-react-widget-table-filter-type';

import './xl-react-widget-table-filters.less';

export default class XlReactWidgetTableFilters extends PureComponent {
    static propTypes = {
        config: tableFiltersType,
        filterProps: PropTypes.object,
        onFilterChange: PropTypes.func.isRequired,
        showFilters: PropTypes.bool
    };

    static defaultProps = {showFilters: true};

    handleModelChange = (key) => (model) => {
        const filterModel = R.pipe(
            R.find(R.propEq('key', key)),
            R.prop('model')
        )(this.props.config);
        if (!R.equals(filterModel, model)) {
            this.props.onFilterChange({type: 'single', change: {key, model}});
        }
    };

    getFilterProps = (key) => R.pathOr({}, ['filterProps', key], this.props);

    filters = {
        [xlReactWidgetTableFilterTypes.AUTOCOMPLETE]: ({className, key, label, model, placeholder}) => {
            const componentProps = this.getFilterProps(key);
            const {
                handlers
            } = componentProps;
            return <XlReactWidgetAutocomplete
                className={className}
                handlers={handlers}
                menuHeight={350}
                metadata={{ label: label, placeholder: placeholder }}
                model={model}
                onModelChange={this.handleModelChange(key)}
            />
        },
        [xlReactWidgetTableFilterTypes.INPUT]: ({ className, key, label, model, placeholder }) =>
            <XlReactWidgetString
                className={className}
                inputIconRenderer={() => <i className="xl-icon search-icon"/>}
                metadata={{
                    label: label,
                    placeholder: placeholder
                }}
                model={model}
                onModelChange={this.handleModelChange(key)}
            />,
        [xlReactWidgetTableFilterTypes.TAGS]: ({className, key, label, model, placeholder}) => {
            const componentProps = this.getFilterProps(key);
            const {
                inputType = 'autocomplete',
                objLabel = 'status',
                skin = tagsAutocompleteSkins.COLOR_AND_CHECKBOX,
                handlers,
                tagTransform = R.identity
            } = componentProps;

            return <XlReactWidgetTags
                className={className}
                handlers={handlers}
                inputType={inputType}
                metadata={{
                    label: label,
                    name: 'autocomplete',
                    placeholder: placeholder
                }}
                model={model}
                objLabel={objLabel}
                onModelChange={this.handleModelChange(key)}
                skin={skin}
                tagTransform={tagTransform}
            />
        },
        [xlReactWidgetTableFilterTypes.DATEPICKER]: ({className, dateFormat, key, label, minimumDate, maximumDate, model, placeholder}) => {
            return <XlReactWidgetDate
                className={className}
                dateFormat={dateFormat}
                maximumDate={maximumDate}
                metadata={{
                    label,
                    placeholder
                }}
                minimumDate={minimumDate}
                model={model}
                onModelChange={this.handleModelChange(key)}
            />;
        },
        [xlReactWidgetTableFilterTypes.TOGGLE_BUTTONS]: ({className, key, label, metadata, model, objLabel, objValue, buttonType}) => {
            return <div className="xl-react-components xl-react-components-toggle-filter">
                <XlReactWidgetLabel metadata={{label}} />
                <XlReactToggleButton
                    className={className}
                    metadata={metadata}
                    model={model}
                    objLabel={objLabel}
                    objValue={objValue}
                    type={buttonType}
                    onToggle={this.handleModelChange(key)}
                />
            </div>;
        },
        [xlReactWidgetTableFilterTypes.CUSTOM]: ({key}) => this.getFilterProps(key).render()
    };

    renderColumns = () => {
        const renderFilterPanelColumn = (column, index) =>
            <div className={`filters-panel-column ${index}`} key={`key-${index}`}>
                {column}
            </div>;

        return RU.mapIndexed(renderFilterPanelColumn, buildColumns(this.filters, this.props.config));
    };

    render() {
        return <Collapse in={this.props.showFilters}>
            <div className="xl-react-redux-table-filters-wrapper">
                <div className="xl-react-redux-table-filters-panel">{this.renderColumns()}</div>
            </div>
        </Collapse>;
    }
}
