import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';

import './xl-react-widget-sized-string-component.less';
import {widgetStringValues} from './xl-react-widget-string-enum';

export class XlReactWidgetSizedStringComponent extends PureComponent {
    static propTypes = {
        assignRef: PropTypes.func.isRequired,
        className: PropTypes.string,
        disabled: PropTypes.bool,
        inputClasses: PropTypes.string,
        model: PropTypes.string,
        name: PropTypes.string,
        onBlur: PropTypes.func,
        onFocus: PropTypes.func,
        onModelChange: PropTypes.func,
        placeholder: PropTypes.string,
        required: PropTypes.bool,
        size: PropTypes.string
    };

    getDisplaySizeClass = (size) => ({
        'xl-react-widget-large-sized-string': size === widgetStringValues.LARGE,
        'xl-react-widget-medium-sized-string': size === widgetStringValues.MEDIUM
    });

    getDisplaySizeRows = (size) => size === widgetStringValues.MEDIUM ? 3 : 7;

    render() {
        const {
            assignRef,
            className,
            disabled,
            inputClasses,
            model,
            name,
            onBlur,
            onFocus,
            onModelChange,
            placeholder,
            required,
            size
        } = this.props;

        const sizeClass = classNames(this.getDisplaySizeClass(size));
        const divClassName = classNames(className, sizeClass);
        return <div className={divClassName}>
            <textarea
                className={inputClasses}
                name={name}
                onBlur={onBlur}
                onChange={onModelChange}
                onFocus={onFocus}
                placeholder={placeholder}
                readOnly={disabled}
                ref={assignRef}
                required={required}
                rows={this.getDisplaySizeRows(size)}
                value={model}
            />
        </div>;
    }
}
