import R from 'ramda';
import webpack from 'webpack';
import HtmlWebpackPlugin from 'html-webpack-plugin';
import paths from '../../utils/paths';
import {dependencies} from '../../../package.json';
import common from './webpack.config.common';

export default {
    entry: {
        polyfills: './app/polyfills.js',
        main: [
            'eventsource-polyfill',
            './app/index'
        ],
        vendor: R.keys(dependencies)
    },
    mode: 'development',
    module: {
        rules: [
            {test: require.resolve('jquery'), use: ['expose-loader?jQuery', 'expose-loader?$']},
            {
                test: /\.ya?ml$/,
                use: 'raw-loader'
            },
            {
                exclude: /(node_modules|tmp)/,
                loader: 'babel-loader',
                test: /\.js$/
            },
            ...common.rulesConfig
        ]
    },
    optimization: {
        splitChunks: {
            cacheGroups: {
                commons: {
                    chunks: 'all',
                    name: 'vendor',
                    test: /[\\/]node_modules[\\/]/
                },
                default: false
            }
        }
    },
    output: {
        filename: '[name].js',
        path: paths.ghPagesDir,
        publicPath: ''
    },
    plugins: [
        new webpack.NoEmitOnErrorsPlugin(),
        new HtmlWebpackPlugin({
            filename: 'kitchen-sink.html',
            template: './app/index.ejs',
            title: 'UI Components'
        })
    ]
};
