const rp = require('request-promise');
const fs = require('fs');
const q = require('q');
const R = require('ramda');

const RUN_PID_FILE = '.visualreview-runid.pid';
const acceptedPrecision = 0.2;

exports.afterLaunch = function(exitCode, vr) {
    const deferred = q.defer();
    const host = process.env.SELENIUM_TEST_VR_HOSTNAME;

    if (fs.existsSync(RUN_PID_FILE) && host) {
        fs.readFile(RUN_PID_FILE, (err, data) => {
            if (err) {
                deferred.reject(err);
            } else {
                const run = JSON.parse(data);
                const root = `http://${host}:7000/`;

                rp(`${root}api/runs/${run.run_id}/analysis`).then((response) => {
                    const diffs = JSON.parse(response).diffs;
                    function isNotAcceptedScreenshot(diff) {
                        return diff.status === 'pending' && diff.percentage > acceptedPrecision;
                    }
                    if (R.isEmpty(R.filter(isNotAcceptedScreenshot, diffs))) {
                        deferred.resolve(vr.cleanup(exitCode));
                    } else {
                        const url = `${root}#/${run.project_id}/${run.suite_id}/${run.run_id}/rp`;
                        console.log('Visual Regression Analysis', response);
                        deferred.reject(`You have pending or rejected screenshots. Please check here:${url}`);
                    }
                }).catch((error) => {
                    deferred.reject(error);
                });
            }
        });
    } else {
        deferred.resolve(vr.cleanup(exitCode));
    }
    return deferred.promise;
};
