import Navigation from '../../dsl/common/navigation';
import TestBindings from '../../dsl/common/test-bindings';
import DipDateTime from '../../dsl/component/dip/dip-date-time';
import WidgetDate from '../../dsl/component/widget/widget-date';
import WidgetTime from '../../dsl/component/widget/widget-time';

describe('XL Dip DateTime scenario', () => {
    afterAll(() => TestBindings.resetPristinity());
    beforeAll(() => Navigation.openDipElement('dipDateTime1'));

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual('November 9th 2016, 3:10:00 pm');
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        DipDateTime.expectLabel('Datetime Label');
        DipDateTime.expectDescription('Pick a date and time');

        WidgetDate.expectInputValue('Nov 9, 2016');
        WidgetTime.expectInputValue('3:10 PM');

        WidgetDate.expectInputEnabled();
        WidgetTime.expectInputEnabled();

        WidgetDate.expectDatePickerNotPresent();
        WidgetTime.expectTimePickerNotPresent();
    });

    it('should allow to select new date and time and reflect that in model', () => {
        WidgetDate.clickOnInput();
        WidgetDate.expectDatePickerPresent();
        WidgetDate.selectDay('13');
        WidgetDate.clickOnLabel();
        TestBindings.expectModelToEqual('November 13th 2016, 3:10:00 pm');
        TestBindings.expectPristineToEqual('false');
        TestBindings.expectValidToEqual('true');

        WidgetDate.expectInputValue('Nov 13, 2016');
        WidgetTime.expectInputValue('3:10 PM');
        WidgetTime.expectTimePickerNotPresent();

        WidgetTime.clickOnInput();
        WidgetTime.expectTimePickerPresent();
        WidgetTime.expectHour('3');
        WidgetTime.expectMinute('10');
        WidgetTime.expectPm();
        WidgetTime.plusHour();
        TestBindings.expectPristineToEqual('false');
        WidgetTime.expectInputValue('4:10 PM');
        WidgetTime.expectHour('4');
        WidgetTime.expectMinute('10');
        TestBindings.expectModelToEqual('November 13th 2016, 4:10:00 pm');
        WidgetTime.clickOnLabel();
        WidgetTime.expectTimePickerNotPresent();
    });

    it('should not accept clicks when component is disabled', () => {
        WidgetDate.expectInputEnabled();
        WidgetTime.expectInputEnabled();
        WidgetDate.expectDatePickerNotPresent();
        WidgetTime.expectTimePickerNotPresent();

        TestBindings.toggleDisabled();
        WidgetDate.expectInputDisabled();
        WidgetTime.expectInputDisabled();

        WidgetTime.clickOnInput();
        WidgetTime.expectTimePickerNotPresent();
        WidgetDate.clickOnInput();
        WidgetDate.expectDatePickerNotPresent();

        TestBindings.toggleDisabled();
        WidgetDate.expectInputEnabled();
        WidgetTime.expectInputEnabled();
    });

    it('should behave different in different view modes', () => {
        TestBindings.displayModeDisplay();
        WidgetDate.expectInputDisabled();
        WidgetTime.expectInputDisabled();

        TestBindings.displayModeCreation();
        WidgetDate.expectInputEnabled();
        WidgetTime.expectInputEnabled();

        TestBindings.displayModeEdit();
        WidgetDate.expectInputEnabled();
        WidgetTime.expectInputEnabled();
    });

    it('should show error about incorrect date format', () => {
        TestBindings.resetPristinity();
        DipDateTime.expectDescription('Pick a date and time');
        WidgetDate.setValue('b');
        WidgetDate.clickOnLabel();
        DipDateTime.expectError('Example: Dec 1, 2018');
        TestBindings.expectPristineToEqual('false');
        TestBindings.expectValidToEqual('false');
    });
});
