import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import DipString from '../../../dsl/component/dip/dip-string';

describe('XL Dip String Scenario', () => {
    afterAll(() => TestBindings.resetPristinity());
    beforeAll(() => Navigation.openDipElement('dipString1'));

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual('');
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        DipString.expectPlaceholderToEqual('String placeholder');
        DipString.expectLabel('String me');
        DipString.expectDescription('This is my string field.');
    });

    it('should be disabled when "disabled" option is chosen', () => {
        DipString.expectNotReadonlyInput();
        TestBindings.toggleDisabled();
        DipString.expectReadonlyInput();
        TestBindings.toggleDisabled();
        DipString.expectNotReadonlyInput();
    });

    it('should behave properly in different display modes', () => {
        TestBindings.displayModeDisplay();
        DipString.expectReadonlyInput();
        TestBindings.displayModeCreation();
        DipString.expectNotReadonlyInput();
        TestBindings.displayModeEdit();
        DipString.expectNotReadonlyInput();
    });

    it('should display cross if any text is typed in input field', () => {
        DipString.setValue('s');
        TestBindings.expectModelToEqual('s');
        DipString.expectClearIconPresent();
        TestBindings.expectPristineToEqual('false');
        DipString.clearValue();
        TestBindings.expectPristineToEqual('false');
        DipString.expectClearIconNotPresent();
        TestBindings.expectModelToEqual('');
    });

    it('should handle password revealing', () => {
        TestBindings.resetPristinity();
        DipString.setValue('some');
        DipString.expectShowPasswordIconNotPresent();
        TestBindings.setInputType('password');
        DipString.expectShowPasswordIconPresent();
        TestBindings.resetPristinity();
        DipString.expectShowPasswordIconNotPresent();
        DipString.clearValue();
        DipString.setValue('some');
        DipString.expectShowPasswordIconPresent();
        DipString.clearValue();
        TestBindings.setInputType('text');
    });
});
