import Form from '../../dsl/component/form/form';
import Navigation from '../../dsl/common/navigation';
import TestBindings from '../../dsl/common/test-bindings';

describe('XL Dip Dynamic Form Default Scenario', () => {
    afterAll(() => TestBindings.resetPristinity());
    beforeAll(() => Navigation.openFormElement('dynamicFormDefault'));

    it('should have all fields pristine and valid', () => {
        Form.expectAllFieldsArePristine();
        Form.expectAllFieldsAreValid();
        Form.expectAllFieldsHaveDescription();
    });

    it('should properly display default values', () => {
        Form.expectFieldHasValue('Label component String', 'input', 'component-0');
        Form.expectFieldHasValue('Label component StringWithTypeHint', 'input', 'string');
        Form.expectFieldHasText('Label component StringLarge', 'textarea', 'large string');
        Form.expectFieldHasText('Label component StringMonaco', 'textarea', 'monaco string');
        Form.expectFieldHasValue('Label component Integer', 'input', 22);
        Form.expectDipSetOfStringsHasTags('Label component Set', ['apple', 'grape']);
        Form.expectDipCollectionHasValues('Label component List', ['tangerine', 'mango']);
        Form.expectFieldHasValue('Label component CI', 'input', 'CiName');
        Form.expectDipMapHasEntry('Label component Map String String', 1, 'someKey', 'someValue');
        const setOfCiValues = [
            'Applications/TestApp/1.0',
            'Infrastructure/TomcatHost',
            'Applications/ZooApp/1.0',
            'Applications/ZooApp/2.0'
        ];
        Form.expectDipCollectionHasValues('Label component Set Of CI', setOfCiValues);
        Form.expectDipCollectionHasValues('Label component List Of CI', ['List Item 1']);
        Form.expectDateTimeToEqual('Label component Date Time', 'Nov 9, 2016', '3:10 PM');
    });

    it('should have proper model', () => {
        Form.expectModelToEqual({
            nameComponentString: 'component-0',
            nameComponentStringWithTypeHint: 'string',
            nameComponentStringLarge: 'large string',
            nameComponentStringMonaco: 'monaco string',
            nameComponentInteger: '22',
            nameComponentBoolean: true,
            nameComponentSet: ['apple', 'grape'],
            nameComponentList: [
                'tangerine',
                'mango'
            ],
            nameComponentCi: 'CiName',
            nameStringEnum: 'mario',
            nameRadioEnum: 'mario',
            mapStringStringName: {someKey: 'someValue'},
            nameSetOfCi: [
                'Applications/TestApp/1.0',
                'Infrastructure/TomcatHost',
                'Applications/ZooApp/1.0',
                'Applications/ZooApp/2.0'
            ],
            nameListOfCi: ['List Item 1', 'Application1'],
            datetimeName: 'November 9th 2016, 3:10:00 pm',
            datetimeStringName: 'November 9th 2017, 3:10:00 pm'
        });
        Form.expectCollectionOfCiFieldHasValidOption('Label component Set Of CI', 1);
        Form.expectCollectionOfCiFieldHasInvalidOption('Label component Set Of CI', 4);
        Form.expectCollectionOfCiFieldHasValidOption('Label component List Of CI', 1);
        Form.expectCollectionOfCiFieldHasInvalidOption('Label component List Of CI', 2);
    });

    it('should properly behave in display mode', () => {
        TestBindings.displayModeDisplay();

        Form.expectFieldReadonly('input', 'Label component String');
        Form.expectFieldDisabled('Label component StringWithTypeHint');
        Form.expectFieldReadonly('textarea', 'Label component StringLarge');
        Form.expectFieldDisabled('Label component Integer');
        Form.expectFieldDisabled('Label component Boolean');
        Form.expectDipStringsDisabled('Label component Set');
        Form.expectFieldNotPresent('Label component List', 'input');
        Form.expectFieldDisabled('Label component CI');
        Form.expectFieldDisabled('Label String enum');
        Form.expectFieldDisabled('Label Radio enum');
        Form.expectDipMapDisabled('Label component Map String String');

        TestBindings.displayModeCreation();
    });

    it('should properly behave when form disabled', () => {
        TestBindings.toggleDisabled();

        Form.expectFieldReadonly('input', 'Label component String');
        Form.expectFieldDisabled('Label component StringWithTypeHint');
        Form.expectFieldReadonly('textarea', 'Label component StringLarge');
        Form.expectFieldDisabled('Label component Integer');
        Form.expectFieldDisabled('Label component Boolean');
        Form.expectDipStringsDisabled('Label component Set');
        Form.expectFieldDisabled('Label component List', 'input');
        Form.expectFieldDisabled('Label component CI');
        Form.expectFieldDisabled('Label String enum');
        Form.expectFieldDisabled('Label Radio enum');
        Form.expectDipMapDisabled('Label component Map String String');

        TestBindings.toggleDisabled();
    });
});

