import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import WidgetAutocomplete from '../../../dsl/component/widget/widget-autocomplete';

describe('XL Widget Autocomplete Scenario', () => {
    afterAll(() => TestBindings.resetPristinity());
    beforeAll(() => Navigation.openWidgetElement('widgetAutocomplete1'));

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual('');
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('false');
        WidgetAutocomplete.expectLabel('Autocomplete*');
        WidgetAutocomplete.expectDescription('This is my autocomplete field.');
    });

    it('should be disabled when "disabled" option is chosen', () => {
        WidgetAutocomplete.expectEnabled();
        TestBindings.toggleDisabled();
        WidgetAutocomplete.expectDisabled();
        TestBindings.toggleDisabled();
        WidgetAutocomplete.expectEnabled();
    });

    it('should behave properly in different display modes', () => {
        TestBindings.displayModeDisplay();
        WidgetAutocomplete.expectDisabled();
        TestBindings.displayModeCreation();
        WidgetAutocomplete.expectEnabled();
        TestBindings.displayModeEdit();
        WidgetAutocomplete.expectEnabled();
    });

    it('should display cross if any text is typed in input field', () => {
        WidgetAutocomplete.setValue('1');
        TestBindings.expectPristineToEqual('false');
        WidgetAutocomplete.clearSelection();
        TestBindings.expectPristineToEqual('false');
        WidgetAutocomplete.expectClearSelectionNotPresent();
        WidgetAutocomplete.expectEmptyValue();
    });

    it('should display only filtered items and be able to select options', () => {
        WidgetAutocomplete.setValue('1');
        WidgetAutocomplete.expectSuggestionValue(1, 'Application1');
        WidgetAutocomplete.selectOption(1);
        WidgetAutocomplete.expectSuggestionListNotPresent();
        TestBindings.expectValidToEqual('true');
        TestBindings.expectModelToEqual('Application1');
        WidgetAutocomplete.clearSelection();
        TestBindings.expectModelToEqual('');
    });
});
