import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import WidgetString from '../../../dsl/component/widget/widget-string';

describe('XL Widget String Default Scenario', () => {
    beforeAll(() => Navigation.openWidgetElement('widgetString1'));

    afterAll(() => TestBindings.resetPristinity());

    it('should change and apply default value', () => {
        TestBindings.expectModelToEqual('');
        TestBindings.expectValidToEqual('true');
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectDefaultValueToEqual('');
        TestBindings.expectDefaultValueButtonToBeVisible();
        WidgetString.expectPlaceholderToEqual('String placeholder');
        WidgetString.expectDescription('This is my string field.');
        WidgetString.expectLabel('String me');

        TestBindings.setDefaultValue('V');

        TestBindings.expectModelToEqual('V');
        TestBindings.expectPristineToEqual('true');
        WidgetString.expectPlaceholderToEqual('String placeholder');
        WidgetString.expectLabel('String me');
        WidgetString.expectDescription('This is my string field.');

        WidgetString.setValue('V');
        TestBindings.setDefaultValue('');
    });

    it('should change pristiny of component to false when value is changed', () => {
        WidgetString.setValue('D');
        TestBindings.expectModelToEqual('D');
        TestBindings.expectPristineToEqual('false');

        TestBindings.setDefaultValue('V');

        TestBindings.expectModelToEqual('V');
        TestBindings.expectPristineToEqual('true');
    });

    it('should display cross if any text is typed in input field', () => {
        TestBindings.setDefaultValue('V');

        TestBindings.expectModelToEqual('V');
        WidgetString.expectClearIconPresent();
        TestBindings.expectPristineToEqual('true');
        WidgetString.clearValue();
        WidgetString.expectClearIconNotPresent();
        TestBindings.expectModelToEqual('');

        TestBindings.setDefaultValue('');
    });
});
