import Navigation from '../../../dsl/common/navigation';
import WidgetReduxTable from '../../../dsl/component/widget/widget-redux-table';

describe('XL Widget Table Scenario', () => {
    beforeAll(() => Navigation.openWidgetElement('widgetReduxTable'));

    it('should have correct initial data', () => {
        WidgetReduxTable.expectHeaders(['Firstname', 'Address', 'City', 'State']);
        WidgetReduxTable.expectTotal(10);
        WidgetReduxTable.expectPageSize(2);
        WidgetReduxTable.expectPageSizes([2, 5, 10]);
        WidgetReduxTable.expectRowToBe(1, ['John Doe', '9889 Random Road', 'Los Angeles', 'CA']);
        WidgetReduxTable.expectRowToBe(2, ['John D. Smith', '1623 Some Street', 'San Francisco', 'CA']);
        WidgetReduxTable.expectRowPresent(2);
        WidgetReduxTable.expectRowNotPresent(3);
    });

    it('should be able to navigate through pages', () => {
        WidgetReduxTable.expectRowToBe(1, ['John Doe', '9889 Random Road', 'Los Angeles', 'CA']);
        WidgetReduxTable.goToNextPage();
        WidgetReduxTable.expectRowToBe(1, ['John A. Smith', '1236 Some Street', 'San Francisco', 'CA']);
        WidgetReduxTable.goToPreviousPage();
        WidgetReduxTable.expectRowToBe(1, ['John Doe', '9889 Random Road', 'Los Angeles', 'CA']);
        WidgetReduxTable.goToPage(3);
        WidgetReduxTable.expectRowToBe(1, ['Bob J. Uncle', '9889 Random Road', 'Los Angeles', 'CA']);
        WidgetReduxTable.goToPage(1);
    });

    it('should be able to change page size', () => {
        WidgetReduxTable.changePaginationSize(5);
        WidgetReduxTable.expectRowPresent(5);
        WidgetReduxTable.expectRowNotPresent(6);
        WidgetReduxTable.changePaginationSize(2);
    });

    it('should sort data', () => {
        WidgetReduxTable.sortColumn(1);
        WidgetReduxTable.expectRowToBe(1, ['Bob C. Uncle', '9996 Random Road', 'Los Angeles', 'CA']);
        WidgetReduxTable.sortColumn(2);
        WidgetReduxTable.expectRowToBe(1, ['Bob C. Uncle', '9996 Random Road', 'Los Angeles', 'CA']);
        WidgetReduxTable.sortColumn(3);
        WidgetReduxTable.expectRowToBe(1, ['John A. Smith', '1236 Some Street', 'San Francisco', 'CA']);
        WidgetReduxTable.sortColumn(4);
        WidgetReduxTable.expectRowToBe(1, ['Joan B. Jones', '3271 Another Ave', 'New York', 'NY']);
    });

    it('should filter above the table filter data', () => {
        WidgetReduxTable.expectAboveTableFilterPanel(['First name', 'Address', 'City', 'State']);
        WidgetReduxTable.aboveTableFilterBy('First name', 'ohn');
        WidgetReduxTable.expectTotal(3);
        WidgetReduxTable.aboveTableFilterBy('Address', '9');
        WidgetReduxTable.aboveTableFilterBy('City', 'Los');
        WidgetReduxTable.aboveTableFilterBy('State', 'CA');
        WidgetReduxTable.expectTotal(1);

        WidgetReduxTable.clearAboveTableFilters(['First name', 'Address', 'City', 'State']);
        WidgetReduxTable.expectTotal(10);
    });

    it('should column filter work properly', () => {
        WidgetReduxTable.changeFilterType('column');
        WidgetReduxTable.expectColumnTableFilter(['firstname', 'address', 'city', 'state']);
        WidgetReduxTable.columnFilterBy('firstname', 'john');
        WidgetReduxTable.expectTotal(3);
        WidgetReduxTable.columnFilterBy('address', '988');
        WidgetReduxTable.columnFilterBy('city', 'los');
        WidgetReduxTable.columnFilterBy('state', 'c');
        WidgetReduxTable.expectTotal(1);
        WidgetReduxTable.clearColumnFilters(['firstname', 'address', 'city', 'state']);
        WidgetReduxTable.expectTotal(10);
    });
});
