import R from 'ramda';
import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import WidgetTags from '../../../dsl/component/widget/widget-tags';
import Form from '../../../dsl/component/form/form';

const model = (value) => ({label: value, value});

describe('XL Widget Tags Scenario', () => {
    afterAll(() => TestBindings.resetPristinity());
    beforeAll(() => Navigation.openWidgetElement('widgetTags1'));

    it('should have right initial state', () => {
        Form.expectModelToEqual(R.map(model, ['test']));
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        WidgetTags.expectDescription('Click me.');
        WidgetTags.expectLabel('insert a tag*');
        WidgetTags.expectAddButtonNotPresent();
        WidgetTags.expectTagPresent(1, 'test');
        WidgetTags.expectClearForTagPresent(1);
        WidgetTags.expectTagNotPresent(2);
    });

    it('should allow to add new tags', () => {
        WidgetTags.setValue('n');
        WidgetTags.expectAddButtonPresent();
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        WidgetTags.clickAddButton();
        WidgetTags.expectAddButtonNotPresent();
        Form.expectModelToEqual(R.map(model, ['test', 'n']));
        TestBindings.expectPristineToEqual('false');
        TestBindings.expectValidToEqual('true');
        WidgetTags.expectEmptyInputValue();
        WidgetTags.expectTagPresent(1, 'test');
        WidgetTags.expectClearForTagPresent(1);
        WidgetTags.expectTagPresent(2, 'n');
        WidgetTags.expectClearForTagPresent(2);
        WidgetTags.expectTagNotPresent(3);
    });

    it('should not allow to do anything when component is disabled', () => {
        TestBindings.toggleDisabled();
        WidgetTags.expectTagPresent(1, 'test');
        WidgetTags.expectClearForTagNotPresent(1, 'test');
        WidgetTags.expectTagPresent(2, 'n');
        WidgetTags.expectClearForTagNotPresent(2, 'n');
        WidgetTags.expectInputDisabled();
        TestBindings.toggleDisabled();
        WidgetTags.expectTagPresent(1, 'test');
        WidgetTags.expectClearForTagPresent(1);
        WidgetTags.expectTagPresent(2, 'n');
        WidgetTags.expectClearForTagPresent(2);
        WidgetTags.expectInputEnabled();
    });

    it('should behave different in different view modes', () => {
        TestBindings.displayModeDisplay();
        WidgetTags.expectInputDisabled();
        WidgetTags.expectTagPresent(1, 'test');
        WidgetTags.expectClearForTagNotPresent(1, 'test');
        WidgetTags.expectDescription('Click me.');

        TestBindings.displayModeCreation();
        WidgetTags.expectInputEnabled();
        WidgetTags.expectTagPresent(1, 'test');
        WidgetTags.expectClearForTagPresent(1);
        WidgetTags.expectDescription('Click me.');

        TestBindings.displayModeEdit();
        WidgetTags.expectInputEnabled();
        WidgetTags.expectTagPresent(1, 'test');
        WidgetTags.expectClearForTagPresent(1);
        WidgetTags.expectDescription('Click me.');
    });

    it('should allow to remove tags', () => {
        WidgetTags.removeTag(1);
        WidgetTags.expectTagPresent(1, 'n');
        WidgetTags.expectClearForTagPresent(1);
        WidgetTags.expectTagNotPresent(2);
    });

    it('should be able to expand and collapse when there are more tags', () => {
        for (const letter of ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j']) {
            WidgetTags.setValue(letter);
            WidgetTags.clickTabKey();
        }

        WidgetTags.removeTag(1);
        WidgetTags.expectTagPresent(1, 'a');
        WidgetTags.expectRemainingTagsLabelPresent();
        WidgetTags.expandTagsField();
        WidgetTags.expectRemainingTagsLabelNotPresent();

        WidgetTags.setValue('k');
        WidgetTags.clickTabKey();
        WidgetTags.expectTagPresent(11, 'k');
        WidgetTags.clickTabKey();

        TestBindings.toggleDisabled();
        WidgetTags.expectTagPresent(1, 'a');
        WidgetTags.expectClearForTagNotPresent(1, 'a');
        WidgetTags.expandTagsField();
        WidgetTags.expectClearForTagNotPresent(11, 'k');
        TestBindings.toggleDisabled();
        WidgetTags.setValue('l');
        WidgetTags.collapseTagsField();
        WidgetTags.expectRemainingTagsLabelPresent();
        WidgetTags.expectTagPresent(1, 'a');
        WidgetTags.expectClearForTagPresent(1);

        R.times(() => WidgetTags.removeTag(1), 7);

        WidgetTags.expectRemainingTagsLabelNotPresent();
        Form.expectModelToEqual(R.map(model, ['h', 'i', 'j', 'k', 'l']));
    });
});
