import React from 'react';
import {mount} from 'enzyme';
import {XlReactDipEnum} from '~/app/xl-react-dip/xl-react-dip-enum/xl-react-dip-enum';

describe('XlReactDipEnum-default', () => {
    const defaultProps = {
        metadata: {
            default: 'test-default-val',
            enumValues: ['test1', 'test2', 'test3', 'test-default'],
            label: '',
            required: false
        }
    };

    it('should have all elements present', () => {
        const defaultProps2 = {
            metadata: {
                default: 'test-default-val',
                description: 'test description',
                enumValues: ['test1', 'test2', 'test3', 'test-default'],
                label: 'enum',
                required: false
            }
        };
        const wrapper = mount(<XlReactDipEnum {...defaultProps2}/>);
        expect(wrapper.find('.xl-react-components-label')).toExist();
        expect(wrapper.find('XlReactWidgetAutocomplete')).toExist();
        expect(wrapper.find('XlReactWidgetAutocomplete .xl-react-autocomplete-container')).toExist();
        expect(wrapper.find('.xl-react-components-description')).toExist();
        expect(wrapper.find('XlReactWidgetAutocomplete input')).toExist();
    });

    it('should erase model on clear ("x") button click', () => {
        const onModelChange = jest.fn();
        const wrapper = mount(<XlReactDipEnum {...defaultProps} onModelChange={onModelChange}/>);
        wrapper.setProps({model: 'test'});
        expect(wrapper.find('XlReactWidgetAutocomplete .close-icon')).toExist();
        wrapper.find('XlReactWidgetAutocomplete .close-icon').simulate('click');
        wrapper.setProps({model: ''});
        expect(wrapper.find('XlReactWidgetAutocomplete .close-icon')).not.toExist();
        expect(onModelChange).toHaveBeenCalledWith('');
        expect(onModelChange).toHaveBeenCalled();
    });

    it('should take value from model', () => {
        const wrapper = mount(<XlReactDipEnum {...defaultProps} model="not-default"/>);
        expect(wrapper.prop('model')).toBe('not-default');
    });
});
