import * as service from '~/app/xl-react-dip/xl-react-dip-map-string-string/xl-react-dip-map-string-string-service';

describe('xl-react-dip-map-string-string-service', () => {
    const model = {cat: 1, pony: 2, donkey: 3}; // eslint-disable-line sort-keys

    describe('getModelKeys', () => {
        it('should return keys without sorting', () => {
            expect(service.getModelKeys(model)).toEqual(['cat', 'pony', 'donkey']);
        });
        it('should return sorted keys', () => {
            expect(service.getModelKeys(model, true)).toEqual(['cat', 'donkey', 'pony']);
        });
    });
    describe('filterModelKeys', () => {
        it('should filter keys and preserve the order defined by modelKeys', () => {
            const modelKeys = ['donkey', 'pony', 'cat'];
            expect(service.filterModel(model, modelKeys, 'o')).toEqual(
                [{key: 'donkey', value: 3}, {key: 'pony', value: 2}]
            );
        });
    });
    describe('getModelWithUpdatedKey', () => {
        it('should update a key in the model and return a new result', () => {
            const expectedResult = {zebra: 1, pony: 2, donkey: 3}; // eslint-disable-line sort-keys
            expect(service.getModelWithUpdatedKey(model, 'cat', 'zebra')).toEqual(expectedResult);
        });
        it('should not do any changes if key is not found', () => {
            expect(service.getModelWithUpdatedKey(model, 'dog', 'zebra')).toEqual(model);
        });
    });
    describe('getModelKeysWithUpdatedKey', () => {
        it('should update a value in the model keys and return a new result', () => {
            const modelKeys = ['cat', 'donkey', 'pony'];
            expect(service.getModelKeysWithUpdatedKey(modelKeys, 'cat', 'zebra')).toEqual(['zebra', 'donkey', 'pony']);
        });
        it('should not do any changes if old value is not found', () => {
            const modelKeys = ['cat', 'donkey', 'pony'];
            expect(service.getModelKeysWithUpdatedKey(modelKeys, 'bee', 'wasp')).toEqual(modelKeys);
        });
    });
    describe('tableSortMethod', () => {
        it('should sort values based on table sort', () => {
            expect(service.tableSortComparator('Aaa', 'Zzz')).toEqual(-1);
            expect(service.tableSortComparator('Zzz', 'Aaa')).toEqual(1);
            expect(service.tableSortComparator(undefined, 'Aaa', true)).toEqual(1);
            expect(service.tableSortComparator(undefined, 'Aaa', false)).toEqual(0);
            expect(service.tableSortComparator('Azz', undefined, true)).toEqual(0);
            expect(service.tableSortComparator('Azz', undefined, false)).toEqual(0);
        });
    });
});
