import {
    getNewInputValue,
    testModel,
    computeNewState
} from '~/app/xl-react-widget/xl-react-widget-autocomplete/xl-react-widget-autocomplete-service';

describe('xl-react-widget-autocomplete-service', () => {
    describe('getNewInputValue', () => {
        it('should return "objLabel" property value if objLabel is defined', () => {
            const model = {label: 'label', value: 'value'};
            const objLabel = 'label';

            expect(getNewInputValue(model, objLabel)).toBe(model.label);
            expect(getNewInputValue(model, '123')).toBe(undefined);
        });
    });
    describe('testModel', () => {
        it('should return true if new model is object, is not empty and it is not equal to previous model', () => {
            const model = {color: 'red', label: 'label', value: 'value'};
            const newModel = {color: 'red', label: 'label', value: 'newValue'};

            expect(testModel(model, newModel)).toBe(true);
        });

        it('should return false if new model is not object or it is empty or equal to previous model', () => {
            const model = {color: 'red', label: 'label', value: 'value'};
            const newModel = {color: 'red', label: 'label', value: 'value'};

            expect(testModel(model, 'newModel')).toBe(false);
            expect(testModel(model, {})).toBe(false);
            expect(testModel(model, newModel)).toBe(false);
        });
    });

    describe('computeNewState', () => {
        const setState = jest.fn();
        const component = {setState};

        afterEach(() => {
            jest.clearAllMocks();
        });

        it('should set label as input value if model is object, and models are different', () => {
            const props = {model: {label: 'label', value: 'value'}, objLabel: 'label'};
            const nextProps = {model: {label: 'nextLabel', value: 'nextValue'}, objLabel: 'label'};

            computeNewState(props, nextProps, component);

            expect(setState).toBeCalledWith({inputValue: 'nextLabel'});
        });

        it('should not do anything if model did not change and model is object', () => {
            computeNewState(
                {model: {value: 'aa'}, objLabel: 'value'},
                {model: {value: 'aa'}, objLabel: 'value'},
                component
            );

            expect(setState).not.toBeCalled();
        });

        it('should set model as input value if model is not object', () => {
            computeNewState({model: 'value'}, {model: 'nextValue'}, component);

            expect(setState).toBeCalledWith({inputValue: 'nextValue'});
        });
    });
});
