import {testSaga} from 'redux-saga-test-plan';
import {call} from 'redux-saga/effects';
import {
    paginate,
    paginateSaga
} from '~/app/xl-react-widget/xl-react-widget-redux-table/sagas/xl-react-widget-redux-table-pagination-saga';
import {getTableById} from '~/app/xl-react-widget/xl-react-widget-redux-table/xl-react-widget-table-selectors';
import tableActions
    from '~/app/xl-react-widget/xl-react-widget-redux-table/xl-react-widget-redux-table-actions';
import {
    withSpinner,
    fetchData
} from '~/app/xl-react-widget/xl-react-widget-redux-table/sagas/xl-react-widget-redux-table-common-saga'; // eslint-disable-line max-len

describe('Widget Redux Table Pagination Saga', () => {
    const mockApi = jest.fn();
    const tableId = 'table-1';
    const page = 3;
    const resultsPerPage = 10;
    const prevPaging = {page: 1, resultsPerPage: 10};

    describe('paginateSaga', () => {
        it('should call pagination api with a spinner during waiting a call response', () => {
            const api = mockApi;
            const saga = testSaga(paginateSaga, {api, page, resultsPerPage, tableId});
            saga.next().call(withSpinner, tableId, call(paginate, {api, page, resultsPerPage, tableId}));
            saga.next().isDone();
        });
    });

    describe('paginate', () => {
        it('should set pagination settings and fetch data based on this', () => {
            const api = mockApi;
            const saga = testSaga(paginate, {api, page, resultsPerPage, tableId});
            saga.next().select(getTableById, tableId);
            saga.next({paging: prevPaging}).put(tableActions.setPaging(tableId, page, resultsPerPage));
            saga.next().call(fetchData, tableId, api, {change: {previous: prevPaging, type: 'paging'}});
            saga.next().isDone();
        });
    });
});
