import {xlReactWidgetReduxTableReducer}
    from '~/app/xl-react-widget/xl-react-widget-redux-table/xl-react-widget-redux-table-reducer';
import xlReactWidgetReduxTableActions
    from '~/app/xl-react-widget/xl-react-widget-redux-table/xl-react-widget-redux-table-actions';

describe('xl-react-widget-redux-table-reducer', () => {
    describe('xlReactWidgetReduxTableReducer', () => {
        it('should initialize redux store with initial table data', () => {
            const mockApi = jest.fn();
            expect(xlReactWidgetReduxTableReducer({})({}, xlReactWidgetReduxTableActions.initialize({
                api: mockApi,
                extra: 'bla',
                tableId: 'table-1'
            }))).toEqual({
                'table-1': {
                    data: [],
                    extra: 'bla',
                    filters: undefined,
                    filtersDebounceTime: 500,
                    layout: undefined,
                    loading: undefined,
                    paging: undefined,
                    sorting: undefined,
                    tableId: 'table-1',
                    total: 0,
                    type: '@xl-react-widget-redux-table/INITIALIZE'
                }
            });
        });
    });

    describe('changeLoadingState', () => {
        it('should change loading state of table to display when it is needed a spinner', () => {
            expect(xlReactWidgetReduxTableReducer({})(
                {'table-1': {data: []}},
                xlReactWidgetReduxTableActions.changeLoadingState('table-1', false))).toEqual({
                'table-1': {
                    data: [],
                    loading: false
                }
            });
        });
    });

    describe('show', () => {
        it('should show table data', () => {
            expect(xlReactWidgetReduxTableReducer({})(
                {'table-1': {data: []}},
                xlReactWidgetReduxTableActions.show({data: [{label: 'my-label'}], tableId: 'table-1'}))).toEqual({
                'table-1': {
                    data: [{label: 'my-label'}],
                    tableId: 'table-1',
                    type: '@xl-react-widget-redux-table/SHOW'
                }
            });
        });
    });

    describe('setSorting', () => {
        it('should set new sorting parameters', () => {
            expect(xlReactWidgetReduxTableReducer({})(
                {'table-1': {data: [{label: 'my-label'}]}},
                xlReactWidgetReduxTableActions.setSorting(
                    'table-1',
                    'firstname',
                    'desc'
                ))).toEqual({
                'table-1': {
                    data: [{label: 'my-label'}],
                    sorting: {
                        name: 'firstname',
                        order: 'desc'
                    }
                }
            });
        });
    });

    describe('setPaging', () => {
        it('should set new paging parameters', () => {
            expect(xlReactWidgetReduxTableReducer({})(
                {'table-1': {data: [{label: 'my-label'}]}},
                xlReactWidgetReduxTableActions.setPaging(
                    'table-1',
                    2,
                    5
                ))).toEqual({
                'table-1': {
                    data: [{label: 'my-label'}],
                    paging: {
                        page: 2,
                        resultsPerPage: 5
                    }
                }
            });
        });
    });

    describe('unmount', () => {
        it('should unmount the table', () => {
            expect(xlReactWidgetReduxTableReducer({})(
                {'table-1': {data: [{label: 'my-label'}]}},
                xlReactWidgetReduxTableActions.unmount('table-1'))).toEqual({});
        });
    });

    describe('mergeContext', () => {
        it('should merge context', () => {
            expect(xlReactWidgetReduxTableReducer({})(
                {
                    'table-1': {data: [{label: 'my-label-1'}]},
                    'table-2': {data: [{label: 'my-label-2'}]}
                },
                xlReactWidgetReduxTableActions.mergeContext('table-1',
                    {data: [{label: 'my-label-updated'}]})))
                .toEqual({
                    'table-1': {data: [{label: 'my-label-updated'}]},
                    'table-2': {data: [{label: 'my-label-2'}]}
                });
        });
    });
});
