import {
    extractFilters,
    shouldShowFilters,
    mergeFilters
} from '~/app/xl-react-widget/xl-react-widget-table/xl-react-widget-table-utils';

describe('xl-react-widget-table-utils', () => {
    describe('extractFilters', () => {
        it('should extract value filters and return column-value filter object', () => {
            expect(extractFilters({
                col1: {filterVal: 'value1'},
                col2: {filterVal: 'value2'}
            })).toEqual({
                col1: 'value1',
                col2: 'value2'
            });
        });
    });

    describe('shouldShowFilters', () => {
        it('should return true if props have filter object and onFilterChange defined', () => {
            expect(shouldShowFilters({
                filter: {
                    col1: '',
                    col2: 'value2'
                },
                onFilterChange: jest.fn()
            })).toBe(true);
        });

        it('should return false if props do not have filter object or onFilterChange defined', () => {
            expect(shouldShowFilters({
                filter: undefined,
                onFilterChange: jest.fn()
            })).toBe(false);

            expect(shouldShowFilters({
                filter: {
                    col1: '',
                    col2: 'value2'
                },
                onFilterChange: undefined
            })).toBe(false);

            expect(shouldShowFilters({
                filter: undefined,
                onFilterChange: undefined
            })).toBe(false);
        });
    });

    describe('mergeFilters', () => {
        it('should merge extracted filters with old filters, if extracted filter doesn\'t have specified column its ' +
            'value should be empty string', () => {
            const extractedFilter = {col1: 'myfilter'};
            const filter = {col1: '', col2: 'oldfilter'};

            expect(mergeFilters(extractedFilter, filter)).toEqual({
                col1: 'myfilter',
                col2: ''
            });
        });
    });
});
